@@ -4,7 +4,7 @@
 // Author:      William Osborne - minimal working wxPalmOS port
 // Modified by:
 // Created:     10/12/04
-// RCS-ID:      $Id: menu.cpp,v 1.10 2006/05/18 07:13:12 ABX Exp $
+// RCS-ID:      $Id: menu.cpp 48053 2007-08-13 17:07:01Z JS $
 // Copyright:   (c) William Osborne
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -395,6 +395,15 @@ wxString wxMenuBar::GetLabelTop(size_t pos) const
     return wxMenuItem::GetLabelFromText(m_titles[pos]);
 }
 
+// Gets the original label at the top-level of the menubar
+wxString wxMenuBar::GetMenuLabel(size_t pos) const
+{
+    wxCHECK_MSG( pos < GetMenuCount(), wxEmptyString,
+                 wxT("invalid menu index in wxMenuBar::GetMenuLabel") );
+
+    return m_titles[pos];
+}
+
 // ---------------------------------------------------------------------------
 // wxMenuBar construction
 // ---------------------------------------------------------------------------
