@@ -222,19 +222,18 @@ class LabelBookDemo(wx.Frame):
         else:
             self.imagelist = self.CreateImageList()
 
+        self.EnableChoices(btype)
         style = self.GetBookStyles()
 
         if btype == 0: # it is a labelbook:
-            self.book = LB.LabelBook(self.mainpanel, -1, style=style)
+            self.book = LB.LabelBook(self.mainpanel, -1, agwStyle=style)
             if self.bookdirection.GetSelection() > 1:
                 self.bookdirection.SetSelection(0)
 
             self.SetUserColours()                
 
         else:
-            self.book = LB.FlatImageBook(self.mainpanel, -1, style=style)
-
-        self.EnableChoices(btype)            
+            self.book = LB.FlatImageBook(self.mainpanel, -1, agwStyle=style)
 
         self.book.AssignImageList(self.imagelist)
 
@@ -253,6 +252,7 @@ class LabelBookDemo(wx.Frame):
             self.Thaw()
 
         self.SendSizeEvent()
+        wx.CallAfter(self.book.SetAGWWindowStyleFlag, style)
 
 
     def EnableChoices(self, btype):
@@ -292,7 +292,7 @@ class LabelBookDemo(wx.Frame):
             style |= INB_GRADIENT_BACKGROUND
         if self.border.GetValue():
             style |= INB_BORDER
-        if self.fittext.GetValue():
+        if self.fittext.IsEnabled() and self.fittext.GetValue():
             style |= INB_FIT_LABELTEXT
 
         return style
@@ -333,7 +333,7 @@ class LabelBookDemo(wx.Frame):
     def OnBookOrientation(self, event):
 
         style = self.GetBookStyles()
-        self.book.SetWindowStyleFlag(style)
+        self.book.SetAGWWindowStyleFlag(style)
 
         event.Skip()
 
@@ -341,7 +341,7 @@ class LabelBookDemo(wx.Frame):
     def OnStyle(self, event):
 
         style = self.GetBookStyles()
-        self.book.SetWindowStyleFlag(style)
+        self.book.SetAGWWindowStyleFlag(style)
 
         event.Skip()
 
@@ -352,29 +352,29 @@ class LabelBookDemo(wx.Frame):
         colour = event.GetValue()
 
         if obj == self.background.GetId():
-            self.book.SetColour(INB_TAB_AREA_BACKGROUND_COLOR, colour)
+            self.book.SetColour(INB_TAB_AREA_BACKGROUND_COLOUR, colour)
         elif obj == self.activetab.GetId():
-            self.book.SetColour(INB_ACTIVE_TAB_COLOR, colour)
+            self.book.SetColour(INB_ACTIVE_TAB_COLOUR, colour)
         elif obj == self.tabsborder.GetId():
-            self.book.SetColour(INB_TABS_BORDER_COLOR, colour)
+            self.book.SetColour(INB_TABS_BORDER_COLOUR, colour)
         elif obj == self.textcolour.GetId():
-            self.book.SetColour(INB_TEXT_COLOR, colour)
+            self.book.SetColour(INB_TEXT_COLOUR, colour)
         elif obj == self.activetextcolour.GetId():
-            self.book.SetColour(INB_ACTIVE_TEXT_COLOR, colour)
+            self.book.SetColour(INB_ACTIVE_TEXT_COLOUR, colour)
         else:
-            self.book.SetColour(INB_HILITE_TAB_COLOR, colour)
+            self.book.SetColour(INB_HILITE_TAB_COLOUR, colour)
 
         self.book.Refresh()
 
 
     def SetUserColours(self):
 
-        self.book.SetColour(INB_TAB_AREA_BACKGROUND_COLOR, self.background.GetColour())
-        self.book.SetColour(INB_ACTIVE_TAB_COLOR, self.activetab.GetColour())
-        self.book.SetColour(INB_TABS_BORDER_COLOR, self.tabsborder.GetColour())
-        self.book.SetColour(INB_TEXT_COLOR, self.textcolour.GetColour())
-        self.book.SetColour(INB_ACTIVE_TEXT_COLOR, self.activetextcolour.GetColour())
-        self.book.SetColour(INB_HILITE_TAB_COLOR, self.hilite.GetColour())
+        self.book.SetColour(INB_TAB_AREA_BACKGROUND_COLOUR, self.background.GetColour())
+        self.book.SetColour(INB_ACTIVE_TAB_COLOUR, self.activetab.GetColour())
+        self.book.SetColour(INB_TABS_BORDER_COLOUR, self.tabsborder.GetColour())
+        self.book.SetColour(INB_TEXT_COLOUR, self.textcolour.GetColour())
+        self.book.SetColour(INB_ACTIVE_TEXT_COLOUR, self.activetextcolour.GetColour())
+        self.book.SetColour(INB_HILITE_TAB_COLOUR, self.hilite.GetColour())
 
 
     def OnPageChanging(self, event):
