@@ -2,7 +2,7 @@
 # AQUABUTTON wxPython IMPLEMENTATION
 #
 # Andrea Gavana, @ 07 October 2008
-# Latest Revision: 07 October 2008, 22.00 GMT
+# Latest Revision: 14 Apr 2010, 12.00 GMT
 #
 #
 # TODO List
@@ -23,6 +23,10 @@
 # --------------------------------------------------------------------------------- #
 
 """
+AquaButton is another custom-drawn button class which *approximatively* mimics
+the behaviour of Aqua buttons on the Mac.
+
+
 Description
 ===========
 
@@ -45,8 +49,32 @@ AquaButton has been tested on the following platforms:
   * Windows (Windows XP).
 
 
-Latest Revision: Andrea Gavana @ 07 October 2008, 22.00 GMT
-Version 0.1
+Window Styles
+=============
+
+`No particular window styles are available for this class.`
+
+
+Events Processing
+=================
+
+This class processes the following events:
+
+================= ==================================================
+Event Name        Description
+================= ==================================================
+``wx.EVT_BUTTON`` Process a `wx.wxEVT_COMMAND_BUTTON_CLICKED` event, when the button is clicked. 
+================= ==================================================
+
+
+License And Version
+===================
+
+AquaButton control is distributed under the wxPython license.
+
+Latest Revision: Andrea Gavana @ 14 Apr 2010, 12.00 GMT
+
+Version 0.3
 
 """
 
@@ -60,14 +88,15 @@ CLICK = 2
 class AquaButtonEvent(wx.PyCommandEvent):
     """ Event sent from the Aqua buttons when the button is activated. """
     
-    def __init__(self, eventType, id):
+    def __init__(self, eventType, eventId):
         """
         Default class constructor.
 
-        @param eventType: the event type;
-        @param id: the event id.
+        :param `eventType`: the event type;
+        :param `eventId`: the event identifier.
         """
-        wx.PyCommandEvent.__init__(self, eventType, id)
+        
+        wx.PyCommandEvent.__init__(self, eventType, eventId)
         self.isDown = False
         self.theButton = None
 
@@ -76,7 +105,7 @@ class AquaButtonEvent(wx.PyCommandEvent):
         """
         Sets the event object for the event.
 
-        @param btn: the button object.
+        :param `btn`: the button object.
         """
         
         self.theButton = btn
@@ -97,15 +126,17 @@ class AquaButton(wx.PyControl):
         """
         Default class constructor.
 
-        @param parent: the L{AquaButton} parent.
-        @param id: the button id;
-        @param bitmap: the button bitmap (if any);
-        @param label: the button text label;
-        @param pos: the button position;
-        @param size: the button size;
-        @param style: the button style (unused);
-        @param validator: the validator associated to the button;
-        @param name: the button name.
+        :param `parent`: the L{AquaButton} parent;
+        :param `id`: window identifier. A value of -1 indicates a default value;
+        :param `bitmap`: the button bitmap (if any);
+        :param `label`: the button text label;
+        :param `pos`: the control position. A value of (-1, -1) indicates a default position,
+         chosen by either the windowing system or wxPython, depending on platform;
+        :param `size`: the control size. A value of (-1, -1) indicates a default size,
+         chosen by either the windowing system or wxPython, depending on platform;
+        :param `style`: the button style (unused);
+        :param `validator`: the validator associated to the button;
+        :param `name`: the button name.
         """
 
         wx.PyControl.__init__(self, parent, id, pos, size, style, validator, name)
@@ -154,30 +185,37 @@ class AquaButton(wx.PyControl):
             self._textColour = wx.WHITE
         
 
-    def LightColour(self, color, percent):
+    def LightColour(self, colour, percent):
         """
-        Return light contrast of color. The color returned is from the scale of
-        color -> white. The percent determines how light the color will be.
-        Percent = 100 return white, percent = 0 returns color.
+        Return light contrast of `colour`. The colour returned is from the scale of
+        `colour` -> white.
+
+        :param `colour`: the input colour to be brightened;
+        :param `percent`: determines how light the colour will be. `percent` = 100
+         returns white, `percent` = 0 returns `colour`.
         """
 
-        end_color = wx.WHITE
-        rd = end_color.Red() - color.Red()
-        gd = end_color.Green() - color.Green()
-        bd = end_color.Blue() - color.Blue()
+        end_colour = wx.WHITE
+        rd = end_colour.Red() - colour.Red()
+        gd = end_colour.Green() - colour.Green()
+        bd = end_colour.Blue() - colour.Blue()
         high = 100
 
-        # We take the percent way of the color from color -. white
+        # We take the percent way of the colour from colour -. white
         i = percent
-        r = color.Red() + ((i*rd*100)/high)/100
-        g = color.Green() + ((i*gd*100)/high)/100
-        b = color.Blue() + ((i*bd*100)/high)/100
+        r = colour.Red() + ((i*rd*100)/high)/100
+        g = colour.Green() + ((i*gd*100)/high)/100
+        b = colour.Blue() + ((i*bd*100)/high)/100
 
         return wx.Colour(r, g, b)
 
     
     def OnPaint(self, event):
-        """ Handles the wx.EVT_PAINT event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_PAINT`` event for L{AquaButton}.
+
+        :param `event`: a `wx.PaintEvent` event to be processed.
+        """
 
         dc = wx.BufferedPaintDC(self)
         gc = wx.GraphicsContext.Create(dc)
@@ -212,7 +250,7 @@ class AquaButton(wx.PyControl):
                                            rc2.x+rc2.width, rc2.y+rc2.height,
                                            rc2.width, wx.NamedColour("grey"), wx.WHITE)
 
-        # Create top water color to give "aqua" effect
+        # Create top water colour to give "aqua" effect
         rc3 = wx.Rect(*rc1)
         rc3.Inflate(-5, -5)
         rc3.height = 15
@@ -274,7 +312,13 @@ class AquaButton(wx.PyControl):
 
     
     def GetPath(self, gc, rc, r):
-        """ Returns a rounded GraphicsPath. """
+        """
+        Returns a rounded `wx.GraphicsPath` rectangle.
+
+        :param `gc`: an instance of `wx.GraphicsContext`;
+        :param `rc`: a client rectangle;
+        :param `r`: the radious of the rounded part of the rectangle.
+        """
     
         x, y, w, h = rc
         path = gc.CreatePath()
@@ -284,13 +328,21 @@ class AquaButton(wx.PyControl):
     
 
     def OnSize(self, event):
-        """ Handles the wx.EVT_SIZE event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_SIZE`` event for L{AquaButton}.
+
+        :param `event`: a `wx.SizeEvent` event to be processed.
+        """
 
         self.Invalidate()
 
 
     def OnLeftDown(self, event):
-        """ Handles the wx.EVT_LEFT_DOWN event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_LEFT_DOWN`` event for L{AquaButton}.
+
+        :param `event`: a `wx.MouseEvent` event to be processed.
+        """
 
         if not self.IsEnabled():
             return
@@ -302,7 +354,11 @@ class AquaButton(wx.PyControl):
 
 
     def OnLeftUp(self, event):
-        """ Handles the wx.EVT_LEFT_UP event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_LEFT_UP`` event for L{AquaButton}.
+
+        :param `event`: a `wx.MouseEvent` event to be processed.
+        """
 
         if not self.IsEnabled() or not self.HasCapture():
             return
@@ -324,7 +380,11 @@ class AquaButton(wx.PyControl):
 
 
     def OnMouseEnter(self, event):
-        """ Handles the wx.EVT_ENTER_WINDOW event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_ENTER_WINDOW`` event for L{AquaButton}.
+
+        :param `event`: a `wx.MouseEvent` event to be processed.
+        """
 
         if not self.IsEnabled():
             return
@@ -335,7 +395,11 @@ class AquaButton(wx.PyControl):
 
 
     def OnMouseLeave(self, event):
-        """ Handles the wx.EVT_LEAVE_WINDOW event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_LEAVE_WINDOW`` event for L{AquaButton}.
+
+        :param `event`: a `wx.MouseEvent` event to be processed.
+        """
 
         self._mouseAction = None
         self.Refresh()
@@ -343,7 +407,11 @@ class AquaButton(wx.PyControl):
 
 
     def OnGainFocus(self, event):
-        """ Handles the wx.EVT_SET_FOCUS event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_SET_FOCUS`` event for L{AquaButton}.
+
+        :param `event`: a `wx.FocusEvent` event to be processed.
+        """
         
         self._hasFocus = True
         self.Refresh()
@@ -355,7 +423,11 @@ class AquaButton(wx.PyControl):
 
 
     def OnLoseFocus(self, event):
-        """ Handles the wx.EVT_KILL_FOCUS event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_KILL_FOCUS`` event for L{AquaButton}.
+
+        :param `event`: a `wx.FocusEvent` event to be processed.
+        """
 
         if self._pulseOnFocus:
             self._gammaFactor = 1.0
@@ -367,7 +439,11 @@ class AquaButton(wx.PyControl):
 
 
     def OnKeyDown(self, event):
-        """ Handles the wx.EVT_KEY_DOWN event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_KEY_DOWN`` event for L{AquaButton}.
+
+        :param `event`: a `wx.KeyEvent` event to be processed.
+        """
         
         if self._hasFocus and event.GetKeyCode() == ord(" "):
             self._mouseAction = HOVER
@@ -376,7 +452,11 @@ class AquaButton(wx.PyControl):
 
 
     def OnKeyUp(self, event):
-        """ Handles the wx.EVT_KEY_UP event for L{AquaButton}. """
+        """
+        Handles the ``wx.EVT_KEY_UP`` event for L{AquaButton}.
+
+        :param `event`: a `wx.KeyEvent` event to be processed.
+        """
 
         if self._hasFocus and event.GetKeyCode() == ord(" "):
             self._mouseAction = HOVER
@@ -386,7 +466,13 @@ class AquaButton(wx.PyControl):
 
 
     def OnPulseTimer(self, event):
-        """ Handles the wx.EVT_TIMER event for L{AquaButton} (only when pulseOnFocus is True). """
+        """
+        Handles the ``wx.EVT_TIMER`` event for L{AquaButton}.
+
+        :param `event`: a `wx.TimerEvent` event to be processed.
+
+        :note: This method is only invoked when `pulseOnFocus` is ``True``.
+        """
 
         if not self._storedBitmap.IsOk():
             self._timer.Stop()
@@ -412,6 +498,8 @@ class AquaButton(wx.PyControl):
         """
         Given the current font and bezel width settings, calculate
         and set a good size.
+
+        :param `size`: an instance of `wx.Size`.        
         """
         
         if size is None:
@@ -422,14 +510,18 @@ class AquaButton(wx.PyControl):
     
 
     def AcceptsFocus(self):
-        """Overridden base class virtual."""
+        """
+        Can this window be given focus by mouse click?
+
+        :note: Overridden from `wx.PyControl`.
+        """
         
         return self.IsShown() and self.IsEnabled()
 
 
     def GetDefaultAttributes(self):
         """
-        Overridden base class virtual.  By default we should use
+        Overridden base class virtual. By default we should use
         the same font/colour attributes as the native Button.
         """
         
@@ -438,22 +530,35 @@ class AquaButton(wx.PyControl):
 
     def ShouldInheritColours(self):
         """
-        Overridden base class virtual.  Buttons usually don't inherit
+        Overridden base class virtual. Buttons usually don't inherit
         the parent's colours.
+
+        :note: Overridden from `wx.PyControl`.
         """
         
         return False
     
 
     def Enable(self, enable=True):
-        """ Enables/disables the button. """
+        """
+        Enables/disables the button.
+
+        :param `enable`: ``True`` to enable the button, ``False`` to disable it.
+        
+        :note: Overridden from `wx.PyControl`.
+        """
         
         wx.PyControl.Enable(self, enable)
         self.Refresh()
 
 
     def SetPulseOnFocus(self, pulse):
-        """ Whether to enable the pulsing effect on gaining focus or not. """
+        """
+        Sets whether to enable the pulsing effect on gaining focus or not.
+
+        :param `pulse`: ``True`` to enable pulsing when the L{AquaButton} gains focus,
+         ``False`` to disable this effect.
+        """
 
         if pulse == self._pulseOnFocus:
             return
@@ -470,7 +575,7 @@ class AquaButton(wx.PyControl):
         
     def DoGetBestSize(self):
         """
-        Overridden base class virtual.  Determines the best size of the
+        Overridden base class virtual. Determines the best size of the
         button based on the label and bezel size.
         """
 
@@ -494,7 +599,13 @@ class AquaButton(wx.PyControl):
 
 
     def SetBackgroundColour(self, colour):
-        """ Sets the button colour when the mouse is not hovering on the button. """
+        """
+        Sets the L{AquaButton} background colour.
+
+        :param `colour`: a valid `wx.Colour` object.
+
+        :note: Overridden from `wx.PyControl`.
+        """
 
         wx.PyControl.SetBackgroundColour(self, colour)
         self._backColour = colour
@@ -502,13 +613,21 @@ class AquaButton(wx.PyControl):
 
 
     def GetBackgroundColour(self):
-        """ Returns the button colour when the mouse is not hovering on the button. """
+        """
+        Returns the button colour when the mouse is not hovering on the button.
+
+        :note: Overridden from `wx.PyControl`.
+        """
 
         return self._backColour
     
 
     def SetHoverColour(self, colour):
-        """ Sets the button colour when the mouse is hovering on the button. """
+        """
+        Sets the button colour when the mouse is hovering on the button.
+
+        :param `colour`: a valid `wx.Colour` object.
+        """
 
         self._hoverColour = colour
         self.Invalidate()
@@ -526,7 +645,13 @@ class AquaButton(wx.PyControl):
     
 
     def SetForegroundColour(self, colour):
-        """ Sets the text colour for L{AquaButton}. """
+        """
+        Sets the L{AquaButton} foreground (text) colour.
+
+        :param `colour`: a valid `wx.Colour` object.
+
+        :note: Overridden from `wx.PyControl`.
+        """
 
         wx.PyControl.SetForegroundColour(self, colour)
         self._textColour = colour
@@ -534,7 +659,11 @@ class AquaButton(wx.PyControl):
 
 
     def GetForegroundColour(self):
-        """ Returns the text colour for L{AquaButton}. """
+        """
+        Returns the text colour for L{AquaButton}.
+
+        :note: Overridden from `wx.PyControl`.
+        """
 
         return self._textColour
     
@@ -557,7 +686,7 @@ class AquaButton(wx.PyControl):
         
 
     def Notify(self):
-        """ Actually sends a wx.EVT_BUTTON event to the listener (if any). """
+        """ Actually sends a ``wx.EVT_BUTTON`` event to the listener (if any). """
         
         evt = AquaButtonEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED, self.GetId())
         evt.SetButtonObj(self)
