@@ -2,7 +2,7 @@
 # PYPROGRESS wxPython IMPLEMENTATION
 #
 # Andrea Gavana, @ 03 Nov 2006
-# Latest Revision: 12 May 2009, 15.00 GMT
+# Latest Revision: 14 Apr 2010, 12.00 GMT
 #
 #
 # TODO List
@@ -25,23 +25,29 @@
 # --------------------------------------------------------------------------------- #
 
 """
+PyProgress is similar to `wx.ProgressDialog` in indeterminated mode, but with a
+different gauge appearance and a different spinning behavior.
+
+
 Description
 ===========
 
-PyProgress is similar to wx.ProgressDialog in indeterminated mode, but with a
+PyProgress is similar to `wx.ProgressDialog` in indeterminated mode, but with a
 different gauge appearance and a different spinning behavior. The moving gauge
 can be drawn with a single solid colour or with a shading gradient foreground.
 The gauge background colour is user customizable.
-The bar does not move always from the beginning to the end as in wx.ProgressDialog
+
+The bar does not move always from the beginning to the end as in `wx.ProgressDialog`
 in indeterminated mode, but spins cyclically forward and backward.
+
 Other options include:
 
-  - Possibility to change the proportion between the spinning bar and the
-    entire gauge, so that the bar can be longer or shorter (the default is 20%);
-  - Modifying the number of steps the spinning bar performs before a forward
-    (or backward) loop reverses.
+- Possibility to change the proportion between the spinning bar and the
+  entire gauge, so that the bar can be longer or shorter (the default is 20%);
+- Modifying the number of steps the spinning bar performs before a forward
+  (or backward) loop reverses.
     
-PyProgress can optionally display a Cancel button, and a wx.StaticText which
+PyProgress can optionally display a ``Cancel`` button, and a `wx.StaticText` which
 outputs the elapsed time from the starting of the process.
 
 
@@ -53,14 +59,35 @@ PyProgress has been tested on the following platforms:
   * Linux Ubuntu (Dapper 6.06)
 
 
-License And Version:
+Window Styles
+=============
+
+This class supports the following window styles:
+
+=================== =========== ==================================================
+Window Styles       Hex Value   Description
+=================== =========== ==================================================
+``PD_CAN_ABORT``            0x1 This flag tells the dialog that it should have a ``Cancel`` button which the user may press. If this happens, the next call to `Update()` will return ``False``.
+``PD_APP_MODAL``            0x2 Make the progress dialog modal. If this flag is not given, it is only 'locally' modal - that is the input to the parent window is disabled, but not to the other ones.
+``PD_AUTO_HIDE``            0x4 Causes the progress dialog to disappear from screen as soon as the maximum value of the progress meter has been reached.
+``PD_ELAPSED_TIME``         0x8 This flag tells the dialog that it should show elapsed time (since creating the dialog).
+=================== =========== ==================================================
+
+
+Events Processing
+=================
+
+`No custom events are available for this class.`
+
+
+License And Version
 ===================
 
-PyProgress is freeware and distributed under the wxPython license. 
+PyProgress is distributed under the wxPython license. 
 
+Latest Revision: Andrea Gavana @ 14 Apr 2010, 12.00 GMT
 
-Latest Revision: Andrea Gavana @ 12 May 2009, 15.00 GMT
-Version 0.2
+Version 0.4
 
 """
 
@@ -78,17 +105,38 @@ Finished = 2
 # Margins between gauge and text/button
 LAYOUT_MARGIN = 8
 
+# PyProgress styles
+PD_CAN_ABORT = wx.PD_CAN_ABORT
+""" This flag tells the dialog that it should have a "Cancel" button which the user""" \
+""" may press. If this happens, the next call to `Update()` will return ``False``. """
+PD_APP_MODAL = wx.PD_APP_MODAL
+""" Make the progress dialog modal. If this flag is not given, it is only 'locally'"""\
+""" modal - that is the input to the parent window is disabled, but not to the other ones. """
+PD_AUTO_HIDE = wx.PD_AUTO_HIDE
+""" Causes the progress dialog to disappear from screen as soon as the maximum""" \
+""" value of the progress meter has been reached. """
+PD_ELAPSED_TIME = wx.PD_ELAPSED_TIME
+""" This flag tells the dialog that it should show elapsed time (since creating the dialog). """
 
 # ---------------------------------------------------------------------------- #
 # Class ProgressGauge
 # ---------------------------------------------------------------------------- #
 
 class ProgressGauge(wx.PyWindow):
-    """ This class provides a visual alternative for wx.Gauge."""
+    """ This class provides a visual alternative for `wx.Gauge`."""
     
     def __init__(self, parent, id=wx.ID_ANY, pos=wx.DefaultPosition,
                  size=(-1,30)):
-        """ Default class constructor. """
+        """
+        Default class constructor.
+
+        :param `parent`: parent window. Must not be ``None``;
+        :param `id`: window identifier. A value of -1 indicates a default value;
+        :param `pos`: the control position. A value of (-1, -1) indicates a default position,
+         chosen by either the windowing system or wxPython, depending on platform;
+        :param `size`: the control size. A value of (-1, -1) indicates a default size,
+         chosen by either the windowing system or wxPython, depending on platform.
+        """
 
         wx.PyWindow.__init__(self, parent, id, pos, size, style=wx.SUNKEN_BORDER)
 
@@ -112,8 +160,15 @@ class ProgressGauge(wx.PyWindow):
 
 
     def SetFirstGradientColour(self, colour):
-        """ Sets the first gradient colour. """
+        """
+        Sets the first gradient colour.
 
+        :param `colour`: a valid `wx.Colour` object.
+        """
+
+        if not isinstance(colour, wx.Colour):
+            colour = wx.NamedColour(colour)
+            
         self._firstGradient = colour
         self.Refresh()
 
@@ -125,7 +180,14 @@ class ProgressGauge(wx.PyWindow):
 
 
     def SetSecondGradientColour(self, colour):
-        """ Sets the second gradient colour. """
+        """
+        Sets the second gradient colour.
+
+        :param `colour`: a valid `wx.Colour` object.
+        """
+
+        if not isinstance(colour, wx.Colour):
+            colour = wx.NamedColour(colour)
 
         self._secondGradient = colour
         self.Refresh()
@@ -138,7 +200,14 @@ class ProgressGauge(wx.PyWindow):
 
 
     def SetGaugeBackground(self, colour):
-        """ Sets the gauge background colour. """
+        """
+        Sets the gauge background colour.
+
+        :param `colour`: a valid `wx.Colour` object.
+        """
+
+        if not isinstance(colour, wx.Colour):
+            colour = wx.NamedColour(colour)
 
         self._background = wx.Brush(colour, wx.SOLID)        
 
@@ -147,6 +216,9 @@ class ProgressGauge(wx.PyWindow):
         """
         Sets the number of steps the gauge performs before switching from
         forward to backward (or vice-versa) movement.
+
+        :param `steps`: the number of steps the gauge performs before switching from
+         forward to backward (or vice-versa) movement.
         """
 
         if steps <= 0:
@@ -178,6 +250,9 @@ class ProgressGauge(wx.PyWindow):
         """
         Sets the relative proportion between the sliding bar and the
         whole gauge.
+
+        :param `proportion`: a floating point number representing the relative
+         proportion between the sliding bar and the whole gauge.
         """
 
         if proportion <= 0 or proportion >= 1:
@@ -188,13 +263,23 @@ class ProgressGauge(wx.PyWindow):
 
         
     def OnEraseBackground(self, event):
-        """ Handles the wx.EVT_ERASE_BACKGROUND event for ProgressGauge. """
+        """
+        Handles the ``wx.EVT_ERASE_BACKGROUND`` event for L{ProgressGauge}.
+
+        :param `event`: a `wx.EraseEvent` event to be processed.
+
+        :note: This method is intentionally empty to reduce flicker.        
+        """
 
         pass
 
 
     def OnPaint(self, event):
-        """ Handles the wx.EVT_PAINT event for ProgressGauge. """
+        """
+        Handles the ``wx.EVT_PAINT`` event for L{ProgressGauge}.
+
+        :param `event`: a `wx.PaintEvent` event to be processed.
+        """
 
         dc = wx.BufferedPaintDC(self)
         dc.SetBackground(self._background)
@@ -219,7 +304,15 @@ class ProgressGauge(wx.PyWindow):
         
 
     def DrawProgress(self, dc, xsize, ysize, increment):
-        """ Actually draws the sliding bar. """
+        """
+        Actually draws the sliding bar.
+
+        :param `dc`: an instance of `wx.DC`;
+        :param `xsize`: the width of the whole progress bar;
+        :param `ysize`: the height of the whole progress bar;
+        :param `increment`: a positive value if we are spinning from left to right,
+         a negative one if we are spinning from right to left.
+        """
 
         if increment > 0:
             col1 = self.GetFirstGradientColour()
@@ -261,17 +354,35 @@ class ProgressGauge(wx.PyWindow):
 
 class PyProgress(wx.Dialog):
     """
-    PyProgress is similar to wx.ProgressDialog in indeterminated mode, but with a
+    PyProgress is similar to `wx.ProgressDialog` in indeterminated mode, but with a
     different gauge appearance and a different spinning behavior. The moving gauge
     can be drawn with a single solid colour or with a shading gradient foreground.
     The gauge background colour is user customizable.
-    The bar does not move always from the beginning to the end as in wx.ProgressDialog
+    The bar does not move always from the beginning to the end as in `wx.ProgressDialog`
     in indeterminated mode, but spins cyclically forward and backward.
     """
 
     def __init__(self, parent=None, id=-1, title="", message="",
-                 style=wx.PD_APP_MODAL|wx.PD_AUTO_HIDE):
-        """ Default class constructor. """
+                 agwStyle=wx.PD_APP_MODAL|wx.PD_AUTO_HIDE):
+        """
+        Default class constructor.
+
+        :param `parent`: parent window;
+        :param `id`: window identifier. A value of -1 indicates a default value;
+        :param `title`: dialog title to show in titlebar;
+        :param `message`: message displayed above the progress bar;
+        :param `style`: the dialog style. This can be a combination of the following bits:
+
+         =================== =========== ==================================================
+         Window Styles       Hex Value   Description
+         =================== =========== ==================================================
+         ``PD_CAN_ABORT``            0x1 This flag tells the dialog that it should have a ``Cancel`` button which the user may press. If this happens, the next call to `Update()` will return ``False``.
+         ``PD_APP_MODAL``            0x2 Make the progress dialog modal. If this flag is not given, it is only 'locally' modal - that is the input to the parent window is disabled, but not to the other ones.
+         ``PD_AUTO_HIDE``            0x4 Causes the progress dialog to disappear from screen as soon as the maximum value of the progress meter has been reached.
+         ``PD_ELAPSED_TIME``         0x8 This flag tells the dialog that it should show elapsed time (since creating the dialog).
+         =================== =========== ==================================================
+         
+        """
 
         wx.Dialog.__init__(self, parent, id, title)
         
@@ -281,12 +392,12 @@ class PyProgress(wx.Dialog):
         # we may disappear at any moment, let the others know about it
         self.SetExtraStyle(self.GetExtraStyle()|wx.WS_EX_TRANSIENT)
 
-        self._hasAbortButton = (style & wx.PD_CAN_ABORT)
+        self._hasAbortButton = (agwStyle & wx.PD_CAN_ABORT)
 
         if wx.Platform == "__WXMSW__":
-        # we have to remove the "Close" button from the title bar then as it is
-        # confusing to have it - it doesn't work anyhow
-        # FIXME: should probably have a (extended?) window style for this
+            # we have to remove the "Close" button from the title bar then as it is
+            # confusing to have it - it doesn't work anyhow
+            # FIXME: should probably have a (extended?) window style for this
             if not self._hasAbortButton:
                 self.EnableClose(False)
     
@@ -322,7 +433,7 @@ class PyProgress(wx.Dialog):
 
         nTimeLabels = 0
 
-        if style & wx.PD_ELAPSED_TIME:
+        if agwStyle & wx.PD_ELAPSED_TIME:
         
             nTimeLabels += 1
             self._elapsed = self.CreateLabel("Elapsed time : ", sizer)
@@ -360,7 +471,7 @@ class PyProgress(wx.Dialog):
         self.Bind(wx.EVT_CLOSE, self.OnClose)
         self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy)
         
-        self._windowStyle = style
+        self._agwStyle = agwStyle
         
         self.SetSizerAndFit(sizer)
     
@@ -374,7 +485,7 @@ class PyProgress(wx.Dialog):
     
         self.Centre(wx.CENTER_FRAME|wx.BOTH)
 
-        if style & wx.PD_APP_MODAL:
+        if agwStyle & wx.PD_APP_MODAL:
             self._winDisabler = wx.WindowDisabler(self)
         else:
             if self._parentTop:
@@ -397,7 +508,12 @@ class PyProgress(wx.Dialog):
 
 
     def CreateLabel(self, text, sizer):
-        """ Creates the wx.StaticText that holds the elapsed time label. """
+        """
+        Creates the `wx.StaticText` that holds the elapsed time label.
+
+        :param `text`: the dialog message to be displayed above the gauge;
+        :param `sizer`: the main `wx.BoxSizer` for L{PyProgress}.
+        """
 
         locsizer = wx.BoxSizer(wx.HORIZONTAL)
         dummy = wx.StaticText(self, wx.ID_ANY, text)
@@ -422,7 +538,17 @@ class PyProgress(wx.Dialog):
     # ----------------------------------------------------------------------------
 
     def UpdatePulse(self, newmsg=""):
-        """ Update the progress dialog with a (optionally) new message. """
+        """
+        Updates the dialog, setting the progress bar to the new value and, if given
+        changes the message above it. Returns ``True`` unless the ``Cancel`` button
+        has been pressed.
+
+        If ``False`` is returned, the application can either immediately destroy the
+        dialog or ask the user for the confirmation.
+
+        :param `newmsg`: The new messages for the progress dialog text, if it is
+         empty (which is the default) the message is not changed.
+        """
        
         self._gauge.Update()
         
@@ -439,7 +565,7 @@ class PyProgress(wx.Dialog):
 
         if self._state == Finished:
 
-            if not self._windowStyle & wx.PD_AUTO_HIDE:
+            if not self._agwStyle & wx.PD_AUTO_HIDE:
                 
                 self.EnableClose()
                 
@@ -472,7 +598,11 @@ class PyProgress(wx.Dialog):
 
 
     def SetFirstGradientColour(self, colour):
-        """ Sets the gauge first gradient colour. """
+        """
+        Sets the gauge first gradient colour.
+
+        :param `colour`: a valid `wx.Colour` object.
+        """
 
         self._gauge.SetFirstGradientColour(colour)
 
@@ -484,7 +614,11 @@ class PyProgress(wx.Dialog):
 
 
     def SetSecondGradientColour(self, colour):
-        """ Sets the gauge second gradient colour. """
+        """
+        Sets the gauge second gradient colour.
+
+        :param `colour`: a valid `wx.Colour` object.
+        """
 
         self._gauge.SetSecondGradientColour(colour)
         
@@ -496,7 +630,11 @@ class PyProgress(wx.Dialog):
 
 
     def SetGaugeBackground(self, colour):
-        """ Sets the gauge background colour. """
+        """
+        Sets the gauge background colour.
+
+        :param `colour`: a valid `wx.Colour` object.
+        """
 
         self._gauge.SetGaugeBackground(colour)
 
@@ -505,6 +643,9 @@ class PyProgress(wx.Dialog):
         """
         Sets the number of steps the gauge performs before switching from
         forward to backward (or vice-versa) movement.
+
+        :param `steps`: the number of steps the gauge performs before switching from
+         forward to backward (or vice-versa) movement.
         """
         
         self._gauge.SetGaugeSteps(steps)
@@ -532,13 +673,21 @@ class PyProgress(wx.Dialog):
         """
         Sets the relative proportion between the sliding bar and the
         whole gauge.
+
+        :param `proportion`: a floating point number representing the relative
+         proportion between the sliding bar and the whole gauge.
         """
 
         self._gauge.SetGaugeProportion(proportion)
 
 
     def ShowDialog(self, show=True):
-        """ Show the dialog. """
+        """
+        Show the dialog.
+
+        :param `show`: ``True`` to show the dialog and re-enable all the other windows,
+         ``False`` otherwise.
+        """
 
         # reenable other windows before hiding this one because otherwise
         # Windows wouldn't give the focus back to the window which had
@@ -554,7 +703,13 @@ class PyProgress(wx.Dialog):
     # ----------------------------------------------------------------------------
 
     def OnCancel(self, event):
-        """ Handles the wx.EVT_BUTTON event for the Cancel button. """
+        """
+        Handles the ``wx.EVT_BUTTON`` event for the dialog.
+
+        :param `event`: a `wx.CommandEvent` event to be processed.
+
+        :note: This method handles the ``Cancel`` button press. 
+        """
 
         if self._state == Finished:
         
@@ -579,14 +734,22 @@ class PyProgress(wx.Dialog):
 
 
     def OnDestroy(self, event):
-        """ Handles the wx.EVT_WINDOW_DESTROY event for PyProgress. """
+        """
+        Handles the ``wx.EVT_WINDOW_DESTROY`` event for L{PyProgress}.
+
+        :param `event`: a `wx.WindowDestroyEvent` event to be processed.
+        """
         
         self.ReenableOtherWindows()
         event.Skip()
 
         
     def OnClose(self, event):
-        """ Handles the wx.EVT_CLOSE event for PyProgress. """
+        """
+        Handles the ``wx.EVT_CLOSE`` event for L{PyProgress}.
+
+        :param `event`: a `wx.CloseEvent` event to be processed.
+        """
 
         if self._state == Uncancelable:
         
@@ -609,9 +772,9 @@ class PyProgress(wx.Dialog):
     
 
     def ReenableOtherWindows(self):
-        """ Re-enables the other windows if using wx.WindowDisabler. """
+        """ Re-enables the other windows if using `wx.WindowDisabler`. """
 
-        if self._windowStyle & wx.PD_APP_MODAL:
+        if self._agwStyle & wx.PD_APP_MODAL:
             if hasattr(self, "_winDisabler"):
                 del self._winDisabler
         
@@ -622,7 +785,12 @@ class PyProgress(wx.Dialog):
     
 
     def SetTimeLabel(self, val, label=None):
-        """ Sets the elapsed time label. """
+        """
+        Sets the elapsed time label.
+
+        :param `val`: the elapsed time since the dialog was shown, in seconds;
+        :param `label`: the new message to display in the elapsed time text.
+        """
 
         if label:
         
@@ -636,7 +804,11 @@ class PyProgress(wx.Dialog):
 
     
     def EnableAbort(self, enable=True):
-        """ Enables or disables the Cancel button. """
+        """
+        Enables or disables the ``Cancel`` button.
+
+        :param `enable`: ``True`` to enable the ``Cancel`` button, ``False`` to disable it.
+        """
 
         if self._hasAbortButton:
             if self._btnAbort:
@@ -644,7 +816,11 @@ class PyProgress(wx.Dialog):
 
 
     def EnableClose(self, enable=True):
-        """ Enables or disables the Close button. """
+        """
+        Enables or disables the ``Close`` button.
+
+        :param `enable`: ``True`` to enable the ``Close`` button, ``False`` to disable it.
+        """
         
         if self._hasAbortButton:
             if self._btnAbort:        
@@ -654,7 +830,7 @@ class PyProgress(wx.Dialog):
 
 
     def DisableAbort(self):
-        """ Disables the Cancel button. """
+        """ Disables the ``Cancel`` button. """
 
         self.EnableAbort(False)
 
