@@ -6,7 +6,7 @@
 # Author:      Robin Dunn
 #
 # Created:     9-Dec-1999
-# RCS-ID:      $Id: buttons.py 60304 2009-04-24 05:30:47Z RD $
+# RCS-ID:      $Id: buttons.py 63815 2010-03-31 19:18:54Z RD $
 # Copyright:   (c) 1999 by Total Control Software
 # Licence:     wxWindows license
 #----------------------------------------------------------------------
@@ -119,15 +119,19 @@ class GenButton(wx.PyControl):
         button based on the label and bezel size.
         """
         w, h, useMin = self._GetLabelSize()
-        defSize = wx.Button.GetDefaultSize()
-        width = 12 + w
-        if useMin and width < defSize.width:
-            width = defSize.width
-        height = 11 + h
-        if useMin and height < defSize.height:
-            height = defSize.height
-        width = width + self.bezelWidth - 1
-        height = height + self.bezelWidth - 1
+        if self.style & wx.BU_EXACTFIT:
+            width = w + 2 + 2 * self.bezelWidth + 4 * int(self.useFocusInd)
+            height = h + 2 + 2 * self.bezelWidth + 4 * int(self.useFocusInd)
+        else:
+            defSize = wx.Button.GetDefaultSize()
+            width = 12 + w
+            if useMin and width < defSize.width:
+                width = defSize.width
+            height = 11 + h
+            if useMin and height < defSize.height:
+                height = defSize.height
+            width = width + self.bezelWidth - 1
+            height = height + self.bezelWidth - 1
         return (width, height)
 
 
@@ -153,8 +157,9 @@ class GenButton(wx.PyControl):
     
 
     def Enable(self, enable=True):
-        wx.PyControl.Enable(self, enable)
-        self.Refresh()
+        if enable != self.IsEnabled():
+            wx.PyControl.Enable(self, enable)
+            self.Refresh()
 
 
     def SetBezelWidth(self, width):
@@ -184,16 +189,11 @@ class GenButton(wx.PyControl):
         fr, fg, fb = min(255,r+32), min(255,g+32), min(255,b+32)
         self.faceDnClr = wx.Colour(fr, fg, fb)
         sr, sg, sb = max(0,r-32), max(0,g-32), max(0,b-32)
-        self.shadowPen = wx.Pen(wx.Colour(sr,sg,sb), 1, wx.SOLID)
+        self.shadowPenClr = wx.Colour(sr,sg,sb)
         hr, hg, hb = min(255,r+64), min(255,g+64), min(255,b+64)
-        self.highlightPen = wx.Pen(wx.Colour(hr,hg,hb), 1, wx.SOLID)
+        self.highlightPenClr = wx.Colour(hr,hg,hb)
         self.focusClr = wx.Colour(hr, hg, hb)
 
-        textClr = self.GetForegroundColour()
-        self.focusIndPen  = wx.Pen(textClr, 1, wx.USER_DASH)
-        self.focusIndPen.SetDashes([1,1])
-        self.focusIndPen.SetCap(wx.CAP_BUTT)
-        
         
     def SetBackgroundColour(self, colour):
         wx.PyControl.SetBackgroundColour(self, colour)
@@ -226,18 +226,18 @@ class GenButton(wx.PyControl):
     def DrawBezel(self, dc, x1, y1, x2, y2):
         # draw the upper left sides
         if self.up:
-            dc.SetPen(self.highlightPen)
+            dc.SetPen(wx.Pen(self.highlightPenClr, 1, wx.SOLID))
         else:
-            dc.SetPen(self.shadowPen)
+            dc.SetPen(wx.Pen(self.shadowPenClr, 1, wx.SOLID))
         for i in range(self.bezelWidth):
             dc.DrawLine(x1+i, y1, x1+i, y2-i)
             dc.DrawLine(x1, y1+i, x2-i, y1+i)
 
         # draw the lower right sides
         if self.up:
-            dc.SetPen(self.shadowPen)
+            dc.SetPen(wx.Pen(self.shadowPenClr, 1, wx.SOLID))
         else:
-            dc.SetPen(self.highlightPen)
+            dc.SetPen(wx.Pen(self.highlightPenClr, 1, wx.SOLID))
         for i in range(self.bezelWidth):
             dc.DrawLine(x1+i, y2-i, x2+1, y2-i)
             dc.DrawLine(x2-i, y1+i, x2-i, y2)
@@ -258,13 +258,22 @@ class GenButton(wx.PyControl):
 
     def DrawFocusIndicator(self, dc, w, h):
         bw = self.bezelWidth
-        self.focusIndPen.SetColour(self.focusClr)
-        dc.SetLogicalFunction(wx.INVERT)
-        dc.SetPen(self.focusIndPen)
+        textClr = self.GetForegroundColour()
+        focusIndPen  = wx.Pen(textClr, 1, wx.USER_DASH)
+        focusIndPen.SetDashes([1,1])
+        focusIndPen.SetCap(wx.CAP_BUTT)
+
+        if wx.Platform == "__WXMAC__":
+            dc.SetLogicalFunction(wx.XOR)
+        else:
+            focusIndPen.SetColour(self.focusClr)
+            dc.SetLogicalFunction(wx.INVERT)
+        dc.SetPen(focusIndPen)
         dc.SetBrush(wx.TRANSPARENT_BRUSH)
         dc.DrawRectangle(bw+2,bw+2,  w-bw*2-4, h-bw*2-4)
         dc.SetLogicalFunction(wx.COPY)
 
+        
     def OnPaint(self, event):
         (width, height) = self.GetClientSizeTuple()
         x1 = y1 = 0
@@ -593,9 +602,9 @@ class GenBitmapTextToggleButton(__ToggleMixin, GenBitmapTextButton):
 
 #----------------------------------------------------------------------
 
-class ThemedGenButton(GenButton):
-    " A themed generic button, and base class for the other themed buttons "
 
+class __ThemedMixin:
+    """ Use the native renderer to draw the bezel, also handle mouse-overs"""
     def InitOtherEvents(self):
         self.Bind(wx.EVT_ENTER_WINDOW, self.OnMouse)
         self.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouse)
@@ -616,25 +625,30 @@ class ThemedGenButton(GenButton):
         if self.GetClientRect().Contains(pt):
             state = wx.CONTROL_CURRENT
         wx.RendererNative.Get().DrawPushButton(self, dc, rect, state)
-        
- 
-class ThemedGenBitmapButton(ThemedGenButton, GenBitmapButton):
+    
+
+
+class ThemedGenButton(__ThemedMixin, GenButton):
+    """A themed generic button"""        
+    pass
+
+class ThemedGenBitmapButton(__ThemedMixin, GenBitmapButton):
     """A themed generic bitmap button."""
     pass
 
-class ThemedGenBitmapTextButton(ThemedGenButton, GenBitmapTextButton):
+class ThemedGenBitmapTextButton(__ThemedMixin, GenBitmapTextButton):
     """A themed generic bitmapped button with text label"""
     pass
     
-class ThemedGenToggleButton(ThemedGenButton, GenToggleButton):
+class ThemedGenToggleButton(__ThemedMixin, GenToggleButton):
     """A themed generic toggle button"""
     pass
 
-class ThemedGenBitmapToggleButton(ThemedGenButton, GenBitmapToggleButton):
+class ThemedGenBitmapToggleButton(__ThemedMixin, GenBitmapToggleButton):
     """A themed generic toggle bitmap button"""
     pass
 
-class ThemedGenBitmapTextToggleButton(ThemedGenButton, GenBitmapTextToggleButton):
+class ThemedGenBitmapTextToggleButton(__ThemedMixin, GenBitmapTextToggleButton):
     """A themed generic toggle bitmap button with text label"""
     pass
 
