@@ -31,8 +31,8 @@ StyleText : Perform custom styling on the text as its added, line by line
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: handlers.py 59439 2009-03-08 22:43:06Z CJP $"
-__revision__ = "$Revision: 59439 $"
+__svnid__ = "$Id: handlers.py 63822 2010-04-02 00:57:27Z CJP $"
+__revision__ = "$Revision: 63822 $"
 
 __all__ = ['GetHandlerById', 'GetHandlerByName', 'GetState',
            'SetState', 'DEFAULT_HANDLER']
@@ -47,6 +47,9 @@ import re
 import eclib.outbuff as outbuff
 import syntax.synglob as synglob
 
+# Local Imports
+import launchxml
+
 #-----------------------------------------------------------------------------#
 # Globals
 DEFAULT_HANDLER = 'handler'
@@ -59,7 +62,7 @@ RE_ANSI_BLOCK = re.compile('\[[34][0-9]m*.*?\[m')
 RE_ANSI_END = re.compile(r'\[[0]{0,1}m')
 RE_ANSI_ESC = re.compile('\[[0-9]+m')
 ANSI = {
-        ## Forground colours ##
+        ## Foreground colours ##
         '[30m' : (1, '#000000'),  # Black
         '[31m' : (2, '#FF0000'),  # Red
         '[32m' : (3, '#00FF00'),  # Green
@@ -82,6 +85,11 @@ ANSI = {
         #'[49m' : default
         }
 
+# Style Types
+STYLE_NORMAL = 0
+STYLE_INFO   = 1
+STYLE_ERROR  = 2
+
 # Process Start/Exit Regular Expression
 RE_PROC_SE = re.compile('>{3,3}.*' + os.linesep)
 
@@ -102,6 +110,40 @@ def GetHandlerByName(name):
     else:
         return HANDLERS[0]
 
+def InitCustomHandlers(path):
+    """Init the custom handlers defined in the launch.xml file
+    @param path: path to directory to find the launch xml in
+    @return: bool
+
+    """
+    loaded = False
+    path = os.path.join(path, u'launch.xml')
+    if os.path.exists(path):
+        lxml = launchxml.LaunchXml()
+        lxml.SetPath(path)
+        loaded = lxml.LoadFromDisk()
+
+        if loaded:
+            for hndlr in lxml.GetHandlers().values():
+                HANDLERS[hndlr.GetLangId()] = XmlHandlerDelegate(hndlr)
+    return loaded
+
+def LoadCustomHandler(xml_str):
+    """Load a custom Launch handler an xml string
+    @param xml_str: Launch Xml String
+    @return: bool
+
+    """
+    try:
+        lxml = launchxml.LaunchXml()
+        lxml.LoadFromString(xml_str)
+        for hndlr in lxml.GetHandlers().values():
+            HANDLERS[hndlr.GetLangId()] = XmlHandlerDelegate(hndlr)
+    except:
+        return False
+    else:
+        return True
+
 def GetState():
     """Get a dictionary capturing the current state of all handlers
     @return: dict { handlername : (default, [commands]) }
@@ -162,7 +204,7 @@ class FileTypeHandler(object):
         return sorted(self.commands.items())
 
     def GetDefault(self):
-        """Get the prefered default command
+        """Get the preferred default command
         @return: string
 
         """
@@ -173,7 +215,7 @@ class FileTypeHandler(object):
         command under.
 
         """
-        return os.environ
+        return dict(os.environ)
 
     def GetName(self):
         """Get the name of this handler"""
@@ -223,7 +265,7 @@ class FileTypeHandler(object):
 
     def SetDefault(self, cmd):
         """Set the prefered default command
-        @param cmd: Command alias/path tuple to set as the preffered one
+        @param cmd: Command alias/path tuple to set as the preferred one
         @postcondition: if cmd is not in the saved command list it will be
                         added to that list.
 
@@ -241,11 +283,18 @@ class FileTypeHandler(object):
 
         """
         # Highlight Start and End lines in info style
+        finfo = False
         for info in RE_PROC_SE.finditer(txt):
             sty_s = start + info.start()
             sty_e = start + info.end()
             stc.StartStyling(sty_s, 0xff)
             stc.SetStyling(sty_e - sty_s, outbuff.OPB_STYLE_INFO)
+            finfo = True
+
+        if finfo:
+            return STYLE_INFO, False
+        else:
+            return STYLE_NORMAL, False
 
 #-----------------------------------------------------------------------------#
 
@@ -300,12 +349,13 @@ class BashHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, BashHandler.RE_BASH_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(BashHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, BashHandler.RE_BASH_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(BashHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -353,12 +403,13 @@ class CamlHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style OCaml Information and Error messages from script output."""
-        if _StyleError(stc, start, txt, CamlHandler.RE_CAML_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(CamlHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, CamlHandler.RE_CAML_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(CamlHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -417,12 +468,13 @@ class HaskellHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style GHC Information and Error messages from script output."""
-        if _StyleError(stc, start, txt, HaskellHandler.RE_HASKELL_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(HaskellHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, HaskellHandler.RE_HASKELL_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(HaskellHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -448,12 +500,13 @@ class HaxeHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style haXe output messages"""
-        if _StyleError(stc, start, txt, HaxeHandler.RE_HAXE_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(HaxeHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, HaxeHandler.RE_HAXE_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(HaxeHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -503,6 +556,17 @@ class KornHandler(FileTypeHandler):
 
 #-----------------------------------------------------------------------------#
 
+class STATAHandler(FileTypeHandler):
+    """FileTypeHandler for Stata"""
+    def __init__(self):
+        FileTypeHandler.__init__(self)
+        self.commands = dict(stata='stata', xstata='xstata')
+        self.default = 'stata'
+
+    __name__ = 'Stata'
+
+#-----------------------------------------------------------------------------#
+
 class LatexHandler(FileTypeHandler):
     """FileTypeHandler for LaTex"""
     def __init__(self):
@@ -538,12 +602,13 @@ class LuaHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, LuaHandler.RE_LUA_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(LuaHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, LuaHandler.RE_LUA_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(LuaHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -581,12 +646,13 @@ class NSISHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, NSISHandler.RE_NSIS_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(NSISHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, NSISHandler.RE_NSIS_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(NSISHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -615,12 +681,13 @@ class PhpHandler(FileTypeHandler):
         output.
 
         """
-        if _StyleError(stc, start, txt, PhpHandler.RE_PHP_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(PhpHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, PhpHandler.RE_PHP_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(PhpHandler, self).StyleText(stc, start, txt)[0]
+        return max(sty, err)
 
 #-----------------------------------------------------------------------------#
 
@@ -660,12 +727,13 @@ class PerlHandler(FileTypeHandler):
         output.
 
         """
-        if _StyleError(stc, start, txt, PerlHandler.RE_PERL_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(PerlHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, PerlHandler.RE_PERL_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(PerlHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -703,7 +771,7 @@ class PythonHandler(FileTypeHandler):
     def GetEnvironment(self):
         """Get the environment to run the python script in"""
         if not hasattr(sys, 'frozen') or sys.platform.startswith('win'):
-            proc_env = os.environ.copy()
+            proc_env = super(PythonHandler, self).GetEnvironment()
         else:
             proc_env = dict()
 
@@ -729,13 +797,19 @@ class PythonHandler(FileTypeHandler):
         output.
 
         """
-        if _StyleError(stc, start, txt, PythonHandler.RE_PY_ERROR) or \
-           _StyleError(stc, start, txt, PythonHandler.RE_PL_ERR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(PythonHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, PythonHandler.RE_PY_ERROR)
+        if more:
+            sty, more = _StyleError(stc, start, txt, PythonHandler.RE_PL_ERR)
+
+        if not err:
+            err = sty
+
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(PythonHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -775,12 +849,13 @@ class RubyHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, RubyHandler.RE_RUBY_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(RubyHandler, self).StyleText(stc, start, txt)
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, RubyHandler.RE_RUBY_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(RubyHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -818,13 +893,14 @@ class TCLHandler(FileTypeHandler):
         _OpenToLine(ifile, line, mainw)
 
     def StyleText(self, stc, start, txt):
-        """Style NSIS output messages"""
-        if _StyleError(stc, start, txt, TCLHandler.RE_TCL_ERROR):
-            return
-        else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(TCLHandler, self).StyleText(stc, start, txt)
+        """Style TCL output messages"""
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, TCLHandler.RE_TCL_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(TCLHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 
@@ -851,12 +927,61 @@ class VBScriptHandler(FileTypeHandler):
 
     def StyleText(self, stc, start, txt):
         """Style VBScript Information and Error messages from script output."""
-        if _StyleError(stc, start, txt, VBScriptHandler.RE_VBS_ERROR):
-            return
+        sty = STYLE_NORMAL
+        err, more = _StyleError(stc, start, txt, VBScriptHandler.RE_VBS_ERROR)
+        if err:
+            err = STYLE_ERROR
+        if more:
+            sty = super(VBScriptHandler, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
+
+#-----------------------------------------------------------------------------#
+
+class XmlHandlerDelegate(FileTypeHandler):
+    """Delegate class for creating new filetype handlers from a Launch
+    Xml object.
+
+    """
+    def __init__(self, xmlobj):
+        """Create the object from the Launch XML Object"""
+        FileTypeHandler.__init__(self)
+
+        # Attributes
+        self._xml = xmlobj
+        self._errpat = self._xml.GetErrorPattern()
+        self._hotspot = self._xml.GetHotSpotPattern()
+
+        # FileTypeHandler implementation
+        self.commands = self._xml.GetCommands()
+        self.default = self._xml.GetDefaultCommand()
+        self._name = self._xml.GetLang()
+
+    @property
+    def __name__(self):
+        return self._name
+
+    def HandleHotSpot(self, mainw, outbuffer, line, fname):
+        """Handle hotspot navigation if it has been defined by the custom
+        handler.
+
+        """
+        if self._hotspot is not None:
+            ifile, line = _FindFileLine(outbuffer, line, fname, self._hotspot)
+            _OpenToLine(ifile, line, mainw)
         else:
-            # Highlight Start end lines this is what the
-            # base classes method does.
-            super(VBScriptHandler, self).StyleText(stc, start, txt)
+            return None
+
+    def StyleText(self, stc, start, txt):
+        """Highlight text if an error pattern was defined."""
+        sty = STYLE_NORMAL
+        err = STYLE_NORMAL
+        if self._errpat is not None:
+            err, more = _StyleError(stc, start, txt, self._errpat)
+            if err:
+                err = STYLE_ERROR
+            if more:
+                sty = super(XmlHandlerDelegate, self).StyleText(stc, start, txt)[0]
+        return max(err, sty)
 
 #-----------------------------------------------------------------------------#
 # Handler Object Dictionary
@@ -876,6 +1001,7 @@ HANDLERS = { 0                      : FileTypeHandler(), # Null Handler
             synglob.ID_LANG_HTML    : HTMLHandler(),
             synglob.ID_LANG_INNO    : InnoSetupHandler(),
             synglob.ID_LANG_LATEX   : LatexHandler(),
+            synglob.ID_LANG_STATA   : STATAHandler(),
             synglob.ID_LANG_LUA     : LuaHandler(),
             synglob.ID_LANG_NEWLISP : NewLispHandler(),
             synglob.ID_LANG_NSIS    : NSISHandler(),
@@ -886,7 +1012,6 @@ HANDLERS = { 0                      : FileTypeHandler(), # Null Handler
             synglob.ID_LANG_PYTHON  : PythonHandler(),
             synglob.ID_LANG_R       : RHandler(),
             synglob.ID_LANG_RUBY    : RubyHandler(),
-            synglob.ID_LANG_TEX     : LatexHandler(),
             synglob.ID_LANG_TCL     : TCLHandler(),
             synglob.ID_LANG_VBSCRIPT : VBScriptHandler() }
 
@@ -900,7 +1025,7 @@ def _FindFileLine(outbuffer, line, fname, regex):
     @param outbuffer: OutputBuffer instance
     @param line: in the buffer
     @param fname: Filname that generated the error message
-    @param regex: a regular exression with two groups the first group needs to
+    @param regex: a regular expression with two groups the first group needs to
                   match the filename. The second group needs to match the line
                   number that the error is reporting
 
@@ -916,7 +1041,7 @@ def _FindFileLine(outbuffer, line, fname, regex):
 
     # If not an absolute path then the error is relative to the
     # script that produced this error message.
-    if not os.path.isabs(ifile):
+    if ifile is not None and not os.path.isabs(ifile):
         dname = os.path.split(fname)[0]
         ifile = os.path.join(dname, ifile)
 
@@ -946,15 +1071,20 @@ def _StyleError(stc, start, txt, regex):
     @param start: start of text just added to buffer
     @param txt: text that was just added
     @param regex: regular expression object for matching the errors
-    @return: bool (True if match), (False if no match)
+    @return: (bool errfound, bool more)
 
     """
+    found_err = False
+    more = False
+    sty_e = start
     for group in regex.finditer(txt):
         sty_s = start + group.start()
         sty_e = start + group.end()
         stc.StartStyling(sty_s, 0xff)
         stc.SetStyling(sty_e - sty_s, outbuff.OPB_STYLE_ERROR)
-    else:
-        return False
+        found_err = True
 
-    return True
+    if sty_e != start + len(txt):
+        more = True
+
+    return found_err, more
