@@ -9,8 +9,8 @@
 
 """Launch User Interface"""
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: launch.py 59917 2009-03-29 17:51:59Z CJP $"
-__revision__ = "$Revision: 59917 $"
+__svnid__ = "$Id: launch.py 63427 2010-02-09 00:11:11Z CJP $"
+__revision__ = "$Revision: 63427 $"
 __version__ = "0.1"
 
 #-----------------------------------------------------------------------------#
@@ -27,7 +27,9 @@ import cfgdlg
 import ed_glob
 import util
 from profiler import Profile_Get, Profile_Set
+import ed_txt
 import ed_msg
+import ebmlib
 import eclib
 
 #-----------------------------------------------------------------------------#
@@ -44,6 +46,7 @@ LAUNCH_KEY = 'Launch.Config'
 # Custom Messages
 MSG_RUN_LAUNCH = ('launch', 'run')
 MSG_RUN_LAST = ('launch', 'runlast')
+MSG_LAUNCH_REGISTER = ('launch', 'registerhandler') # msgdata == xml string
 
 # Value request messages
 REQUEST_ACTIVE = 'Launch.IsActive'
@@ -56,6 +59,18 @@ def OnStoreConfig(msg):
     Profile_Set(LAUNCH_KEY, handlers.GetState())
 ed_msg.Subscribe(OnStoreConfig, cfgdlg.EDMSG_LAUNCH_CFG_EXIT)
 
+def OnRegisterHandler(msg):
+    """Register a custom handler
+    @param msg: dict(xml=xml_str, loaded=bool)
+
+    """
+    data = msg.GetData()
+    loaded = False
+    if isinstance(data, dict) and 'xml' in data:
+        loaded = handlers.LoadCustomHandler(data['xml'])
+    data['loaded'] = loaded
+ed_msg.Subscribe(OnRegisterHandler, MSG_LAUNCH_REGISTER)
+
 #-----------------------------------------------------------------------------#
 
 class LaunchWindow(eclib.ControlBox):
@@ -64,6 +79,7 @@ class LaunchWindow(eclib.ControlBox):
         eclib.ControlBox.__init__(self, parent)
 
         # Attributes
+        self._log = wx.GetApp().GetLog()
         self._mw = self.__FindMainWindow()
         self._buffer = OutputDisplay(self)
         self._fnames = list()
@@ -80,12 +96,16 @@ class LaunchWindow(eclib.ControlBox):
 
         # Setup
         self.__DoLayout()
+        if not handlers.InitCustomHandlers(ed_glob.CONFIG['CACHE_DIR']):
+            util.Log(u"[launch][warn] failed to load launch extensions")
+
         hstate = Profile_Get(LAUNCH_KEY)
         if hstate is not None:
             handlers.SetState(hstate)
         if self._prefs is None:
             Profile_Set(cfgdlg.LAUNCH_PREFS,
                         dict(autoclear=False,
+                             errorbeep=False,
                              defaultf=self._buffer.GetDefaultForeground().Get(),
                              defaultb=self._buffer.GetDefaultBackground().Get(),
                              errorf=self._buffer.GetErrorForeground().Get(),
@@ -96,6 +116,7 @@ class LaunchWindow(eclib.ControlBox):
                              warnb=self._buffer.GetWarningBackground().Get()))
             self._prefs = Profile_Get(cfgdlg.LAUNCH_PREFS)
 
+        self._buffer.SetPrefs(self._prefs)
         self.UpdateBufferColors()
         cbuffer = self._mw.GetNotebook().GetCurrentCtrl()
         self.SetupControlBar(cbuffer)
@@ -154,7 +175,7 @@ class LaunchWindow(eclib.ControlBox):
 
         # Script Label
         ctrlbar.AddControl((5, 5), wx.ALIGN_LEFT)
-        self._chFiles = wx.Choice(ctrlbar, choices=[''])
+        self._chFiles = wx.Choice(ctrlbar, wx.ID_ANY)#, choices=[''])
         ctrlbar.AddControl(self._chFiles, wx.ALIGN_LEFT)
 
         # Args
@@ -299,15 +320,12 @@ class LaunchWindow(eclib.ControlBox):
         Profile_Set(LAUNCH_KEY, handlers.GetState())
         self.UpdateBufferColors()
 
+    @ed_msg.mwcontext
     def OnFileOpened(self, msg):
-        """Reset state when a file open message is recieved
+        """Reset state when a file open message is received
         @param msg: Message Object
 
         """
-        # Only update when in the active window
-        if not self._mw.IsActive():
-            return
-
         # Update the file choice control
         self._config['lang'] = GetLangIdFromMW(self._mw)
         self.UpdateCurrentFiles(self._config['lang'])
@@ -318,6 +336,7 @@ class LaunchWindow(eclib.ControlBox):
         # Setup filetype settings
         self.RefreshControlBar()
 
+    @ed_msg.mwcontext
     def OnLexerChange(self, msg):
         """Update the status of the currently associated file
         when a file is saved. Used for updating after a file type has
@@ -325,8 +344,8 @@ class LaunchWindow(eclib.ControlBox):
         @param msg: Message object
 
         """
-        if not self._mw.IsActive():
-            return
+        self._log("[launch][info] Lexer changed handler - context %d" %
+                  self._mw.GetId())
 
         mdata = msg.GetData()
         # For backwards compatibility with older message format
@@ -334,7 +353,10 @@ class LaunchWindow(eclib.ControlBox):
             return
 
         fname, ftype = msg.GetData()
-        if ftype != self._config['lang']:
+        # Update regardless of whether lexer has changed or not as the buffer
+        # may have the lexer set before the file was saved to disk.
+        if fname:
+            #if ftype != self._config['lang']:
             self.UpdateCurrentFiles(ftype)
             self.SetControlBarState(fname, ftype)
 
@@ -441,6 +463,7 @@ class LaunchWindow(eclib.ControlBox):
             self._isready = False
             for ctrl in (exe_ch, args_txt, run_btn, self._chFiles):
                 ctrl.Disable()
+            self._chFiles.Clear()
 
     def Run(self, fname, cmd, args, ftype):
         """Run the given file
@@ -461,6 +484,11 @@ class LaunchWindow(eclib.ControlBox):
 
         handler = handlers.GetHandlerById(ftype)
         path, fname = os.path.split(fname)
+        if wx.Platform == '__WXMSW__':
+            fname = u"\"" + fname + u"\""
+        else:
+            fname = fname.replace(u' ', u'\\ ')
+
         self._worker = eclib.ProcessThread(self._buffer,
                                            cmd, fname,
                                            args, path,
@@ -592,9 +620,10 @@ class LaunchWindow(eclib.ControlBox):
 
         items = [ os.path.basename(fname) for fname in self._fnames ]
         try:
-            self._chFiles.SetItems(items)
-            if len(self._fnames):
-                self._chFiles.SetToolTipString(self._fnames[0])
+            if len(u''.join(items)):
+                self._chFiles.SetItems(items)
+                if len(self._fnames):
+                    self._chFiles.SetToolTipString(self._fnames[0])
         except TypeError:
             util.Log("[Launch][err] UpdateCurrent Files: " + str(items))
             self._chFiles.SetItems([''])
@@ -610,6 +639,7 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
         # Attributes
         self._mw = parent.GetMainWindow()
         self._cfile = ''
+        self._prefs = dict()
 
         # Setup
         font = Profile_Get('FONT1', 'font', wx.Font(11, wx.FONTFAMILY_MODERN,
@@ -625,7 +655,11 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
 
         """
         handler = self.GetCurrentHandler()
-        handler.StyleText(self, start, txt)
+        style = handler.StyleText(self, start, txt)
+
+        # Ring the bell if there was an error and option is enabled
+        if style == handlers.STYLE_ERROR and self._prefs.get('errorbeep', False):
+            wx.Bell()
 
     def DoFilterInput(self, txt):
         """Filter the incoming input
@@ -647,6 +681,26 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
         handler.HandleHotSpot(self._mw, self, line, fname)
         self.GetParent().SetupControlBar(GetTextBuffer(self._mw))
 
+    def DoProcessError(self, code, excdata=None):
+        """Handle notifications of when an error occurs in the process
+        @param code: an OBP error code
+        @keyword excdata: Exception string
+        @return: None
+
+        """
+        if code == eclib.OPB_ERROR_INVALID_COMMAND:
+            self.AppendUpdate(_("The requested command could not be executed.") + u"\n")
+
+        # Log the raw exception data to the log as well
+        if excdata is not None:
+            try:
+                excstr = str(excdata)
+                if not ebmlib.IsUnicode(excstr):
+                    excstr = ed_txt.DecodeString(excstr)
+                util.Log(u"[launch][err] %s" % excdata)
+            except UnicodeDecodeError:
+                util.Log(u"[launch][err] error decoding log message string")
+
     def DoProcessExit(self, code=0):
         """Do all that is needed to be done after a process has exited"""
         self.AppendUpdate(">>> %s: %d%s" % (_("Exit Code"), code, os.linesep))
@@ -667,6 +721,13 @@ class OutputDisplay(eclib.OutputBuffer, eclib.ProcessBufferMixin):
         handler = handlers.GetHandlerById(lang_id)
         return handler
 
+    def SetPrefs(self, prefs):
+        """Set the launch prefs
+        @param prefs: dict
+
+        """
+        self._prefs = prefs
+
 #-----------------------------------------------------------------------------#
 def GetLangIdFromMW(mainw):
     """Get the language id of the file in the current buffer
