@@ -18,8 +18,8 @@ element is defined in the file.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cbrowser.py 60379 2009-04-26 05:24:41Z CJP $"
-__revision__ = "$Revision: 60379 $"
+__svnid__ = "$Id: cbrowser.py 63308 2010-01-30 00:03:47Z CJP $"
+__revision__ = "$Revision: 63308 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -53,7 +53,7 @@ STRINGS = ( _("Class Definitions"), _("Defines"), _("Function Definitions"),
             _("Sections"), _("Style Tags"), _("Subroutines"),
             _("Subroutine Declarations"), _("Task Definitions"), 
             _("Modules"), _("Functions"), _("Public Functions"),
-            _("Public Subroutines") )
+            _("Public Subroutines"), _("Imports") )
 del STRINGS
 
 #--------------------------------------------------------------------------#
@@ -65,6 +65,7 @@ class CodeBrowserTree(wx.TreeCtrl):
         wx.TreeCtrl.__init__(self, parent, id, pos, size, style)
 
         # Attributes
+        self._log = wx.GetApp().GetLog()
         self._mw = parent
         self._menu = None
         self._selected = None
@@ -74,7 +75,11 @@ class CodeBrowserTree(wx.TreeCtrl):
         self.icons = dict()
         self.il = None
 
+        # struct used in buffer-tree sync
+        self._ds_flat = list() # list of tuples - [(line, item_id),(line, item_id)...]
+
         self._timer = wx.Timer(self)
+        self._sync_timer = wx.Timer(self)
         self._cpage = None
         self._force = False
 
@@ -95,12 +100,17 @@ class CodeBrowserTree(wx.TreeCtrl):
         self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnActivated)
         self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnContext)
         self.Bind(wx.EVT_MENU, self.OnMenu)
-        self.Bind(wx.EVT_TIMER, self.OnStartJob)
+        self.Bind(wx.EVT_TIMER, self.OnStartJob, self._timer)
+        self.Bind(wx.EVT_TIMER, lambda evt: self._SyncTree(), self._sync_timer)
         self.Bind(EVT_JOB_FINISHED, self.OnTagsReady)
+
+        # Editra Message Handlers
         ed_msg.Subscribe(self.OnThemeChange, ed_msg.EDMSG_THEME_CHANGED)
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_UI_NB_CHANGED)
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_FILE_OPENED)
         ed_msg.Subscribe(self.OnUpdateTree, ed_msg.EDMSG_FILE_SAVED)
+        ed_msg.Subscribe(self.OnSyncTree, ed_msg.EDMSG_UI_STC_POS_CHANGED)
+        ed_msg.Subscribe(self.OnEditorRestore, ed_msg.EDMSG_UI_STC_RESTORE)
 
         # Backwards compatibility
         if hasattr(ed_msg, 'EDMSG_UI_STC_LEXER') and \
@@ -113,11 +123,61 @@ class CodeBrowserTree(wx.TreeCtrl):
         ed_msg.Unsubscribe(self.OnUpdateTree)
         ed_msg.Unsubscribe(self.OnThemeChange)
         ed_msg.Unsubscribe(self.OnUpdateFont)
+        ed_msg.Unsubscribe(self.OnSyncTree)
+        ed_msg.Unsubscribe(self.OnEditorRestore)
 
     def _GetCurrentCtrl(self):
         """Get the current buffer"""
         return self._mw.GetNotebook().GetCurrentCtrl()
 
+    def _GetFQN(self, node):
+        """Returns fully qualified name of the tree node in the form
+        ...grandparent.parent.child
+        @param node: node id
+
+        """
+        rval=""
+        if node and node.IsOk():
+            path = [self.GetItemText(node)]
+            parent = self.GetItemParent(node)
+            root = self.GetRootItem()
+            while parent and parent.IsOk() and parent != root:
+                path.append(self.GetItemText(parent))
+                parent = self.GetItemParent(parent)
+            
+            # Join path elements and strip line information as this could be changed
+            path.reverse()
+            rval = '.'.join([p.split(u'[')[0].strip() for p in path])
+            
+        return rval
+    
+    def _ClearTree(self):
+        """Clear the tree and caches"""
+        self._cdoc = None
+        self._ds_flat = list()
+        self.DeleteChildren(self.root)
+
+    def _FindNodeForLine(self, line):
+        """Returns node id of the docstruct element given line belongs to
+        @param line: line number
+        @returns: tree node id
+
+        """
+        # HACK This should probably be done with bisect search
+        rval = None
+        line += 1
+        if self._ds_flat:
+            for citem in self._ds_flat:
+                if citem[0] < line:
+                    cline = citem[0]
+                    rval = citem[1]
+                else:
+                    break
+
+#        self._log("[codebrowser][info] For line %d found item %s" % \
+#                  (line, self._GetFQN(rval)))
+        return rval
+
     def _SetupImageList(self):
         """Setup the image list for the tree"""
         imglst = wx.ImageList(16, 16)
@@ -153,15 +213,38 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         """
         img = self.icons.get(cobj.type, None)
+
+        # Try and find an appropriate fallback icon
         if img is None:
             if isinstance(cobj, taglib.Function):
                 img = self.icons['function']
+            elif isinstance(cobj, taglib.Class):
+                img = self.icons['class']
             elif isinstance(cobj, taglib.Scope):
                 img = self.icons['section']
             else:
                 img = self.icons['variable']
         return img
 
+    def _SyncTree(self):
+        """Synchronize tree node selection with current position in the text
+        """
+        if not self._ShouldUpdate():
+            return
+        line = self._GetCurrentCtrl().GetCurrentLineNum()
+        self._log("[codebrowser][info] Syncing tree for position %d" % line)
+        scope_item = self._FindNodeForLine(line)
+        if scope_item:
+            selected = self.GetSelection()
+            if selected:
+                if selected != scope_item:
+                    self.Unselect()
+                    self.SelectItem(scope_item)
+            else:
+                self.SelectItem(scope_item)
+                
+            self.EnsureVisible(scope_item)
+
     def _ShouldUpdate(self):
         """Check whether the tree should do an update or not
         @return: bool
@@ -179,7 +262,10 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         """
         if self.nodes.get('classes', None) is None:
-            croot = self.AppendItem(self.GetRootItem(), _("Class Definitions"))
+            desc = self._cdoc.GetElementDescription('class')
+            if desc == 'class':
+                desc = "Class Definitions"
+            croot = self.AppendItem(self.GetRootItem(), _(desc))
             self.SetItemHasChildren(croot)
             self.SetPyData(croot, None)
             self.SetItemImage(croot, self.icons['class'])
@@ -195,6 +281,7 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         """
         item_id = self.AppendItem(node, u"%s [%d]" % (cobj.GetName(), 1 + cobj.GetLine()), img)
+        self._ds_flat.append((cobj.GetLine(), item_id))
         self.SetPyData(item_id, cobj.GetLine())
         # If the item is a scope it may have sub items
         if isinstance(cobj, taglib.Scope):
@@ -251,6 +338,10 @@ class CodeBrowserTree(wx.TreeCtrl):
         for key in self.nodes.keys():
             self.nodes[key] = None
 
+    def GetMainWindow(self):
+        """Get this panels main window"""
+        return self._mw
+
     def GotoElement(self, tree_id):
         """Navigate the cursor to the element identified in the
         code browser tree.
@@ -261,7 +352,10 @@ class CodeBrowserTree(wx.TreeCtrl):
         if line is not None:
             ctrl = self._mw.GetNotebook().GetCurrentCtrl()
             ctrl.GotoLine(line)
-            ctrl.SetFocus()
+
+            # Need CallAfter to make the focus work on Windows
+            # Otherwise the tree control will reclaim the focus
+            wx.CallAfter(ctrl.SetFocus)
 
     def OnActivated(self, evt):
         """Handle when an item is clicked on
@@ -313,6 +407,11 @@ class CodeBrowserTree(wx.TreeCtrl):
         else:
             evt.Skip()
 
+    @ed_msg.mwcontext
+    def OnEditorRestore(self, msg):
+        """Called when editor size is unmaximized"""
+        self.OnUpdateTree()
+
     def OnThemeChange(self, msg):
         """Update the images when Editra's theme changes
         @param msg: Message Object
@@ -356,8 +455,24 @@ class CodeBrowserTree(wx.TreeCtrl):
         pane = self._mw.GetFrameManager().GetPane(PANE_NAME)
         evt.Check(pane.IsShown())
 
+    @ed_msg.mwcontext
+    def OnSyncTree(self, msg):
+        """Handler for tree synchronization.
+        Uses a one shot timer to optimize multiple fast caret movement events.
+
+        """
+        if not self._ShouldUpdate():
+            return # If panel is not visible don't update
+        
+        if self._sync_timer.IsRunning():
+            self._sync_timer.Stop()
+        
+        #One shot timer for tree sync
+        self._sync_timer.Start(300, True)
+
     def OnStartJob(self, evt):
         """Start the tree update job
+        
         @param evt: wxTimerEvent
 
         """
@@ -389,8 +504,7 @@ class CodeBrowserTree(wx.TreeCtrl):
                                   StringIO.StringIO(self._cpage.GetText()))
             wx.CallLater(75, thread.start)
         else:
-            self._cdoc = None
-            self.DeleteChildren(self.root)
+            self._ClearTree()
             ed_msg.PostMessage(ed_msg.EDMSG_PROGRESS_SHOW,
                                (self._mw.GetId(), False))
             return
@@ -401,9 +515,17 @@ class CodeBrowserTree(wx.TreeCtrl):
         @keyword force: Force update
 
         """
-        # Don't update when this window is not Active
-        istop = wx.GetApp().GetTopWindow() == self._mw
-        if not force and not self._mw.IsActive() and not istop:
+        # Check for if update should be skipped
+        if not force:
+            context = None
+            if msg is not None:
+                context = msg.GetContext()
+
+            if context is not None and context != self._mw.GetId():
+                return
+        
+        # Don't update if panel is not visible
+        if not self._ShouldUpdate():
             return
 
         page = self._GetCurrentCtrl()
@@ -411,8 +533,7 @@ class CodeBrowserTree(wx.TreeCtrl):
 
         # If its a blank document just clear out
         if not len(cfname):
-            self._cdoc = None
-            self.DeleteChildren(self.root)
+            self._ClearTree()
             return
 
         # If document job is same as current don't start a new one
@@ -427,7 +548,7 @@ class CodeBrowserTree(wx.TreeCtrl):
 
             # Start the oneshot timer for beginning the tag generator job
             self._timer.Start(300, True)
-
+    
     def OnShowBrowser(self, evt):
         """Show the browser pane
         @param evt: wx.MenuEvent
@@ -447,8 +568,8 @@ class CodeBrowserTree(wx.TreeCtrl):
         @param tags: DocStruct object
 
         """
+        self._ClearTree()
         self._cdoc = tags
-        self.DeleteChildren(self.root)
         # Check and add any common types in the document first
 
         # Global Variables
@@ -470,6 +591,9 @@ class CodeBrowserTree(wx.TreeCtrl):
         for node in [ node for node in self.nodes.values()
                       if node is not None and node != self.nodes['globals']]:
             self.Expand(node)
+        
+        self._ds_flat.sort(cmp=lambda x,y: cmp(x[0],y[0]))
+        self._SyncTree()
 
 #--------------------------------------------------------------------------#
 # Tag Generator Thread
