@@ -12,8 +12,8 @@ Provides helper functions and classes for managing documents and their services.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: doctools.py 60537 2009-05-07 02:39:20Z CJP $"
-__revision__ = "$Revision: 60537 $"
+__svnid__ = "$Id: doctools.py 63349 2010-02-01 22:13:57Z CJP $"
+__revision__ = "$Revision: 63349 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -23,6 +23,7 @@ import sys
 # Editra Libraries
 import util
 from profiler import Profile_Get
+import ebmlib
 
 #--------------------------------------------------------------------------#
 
@@ -34,6 +35,8 @@ class DocPositionMgr(object):
     @note: saves config to ~/.Editra/cache/
 
     """
+    _poscache = ebmlib.HistoryCache(100)
+
     def __init__(self):
         """Creates the position manager object
 
@@ -45,10 +48,6 @@ class DocPositionMgr(object):
         self._book = None
         self._records = dict()
 
-        # Non persisted navigation cache
-        self._poscache = list()
-        self._cpos = -1
-
     def InitPositionCache(self, book_path):
         """Initialize and load the on disk document position cache.
         @param book_path: path to on disk cache
@@ -59,18 +58,20 @@ class DocPositionMgr(object):
         if Profile_Get('SAVE_POS'):
             self.LoadBook(book_path)
 
-    def AddNaviPosition(self, fname, pos):
+    @classmethod
+    def AddNaviPosition(cls, fname, pos):
         """Add a new postion to the navigation cache
         @param fname: file name
         @param pos: position
 
         """
-        clen = len(self._poscache)
-        if self._cpos+1 != clen:
-            self._poscache = self._poscache[:self._cpos]
+        # Don't put two identical positions in the cache next to eachother
+        pre = cls._poscache.PeekPrevious()
+        next = cls._poscache.PeekNext()
+        if (fname, pos) in (pre, next):
+            return
 
-        self._poscache.append((fname, pos))
-        self._cpos = len(self._poscache) - 1
+        cls._poscache.PutItem((fname, pos))
 
     def AddRecord(self, vals):
         """Adds a record to the dictionary from a list of the
@@ -85,59 +86,63 @@ class DocPositionMgr(object):
         else:
             return False
 
+    @classmethod
+    def CanNavigateNext(cls):
+        """Are there more cached navigation positions?
+        @return: bool
+
+        """
+        return cls._poscache.HasNext()
+
+    @classmethod
+    def CanNavigatePrev(cls):
+        """Are there previous cached navigation positions?
+        @return: bool
+
+        """
+        return cls._poscache.HasPrevious()
+
+    @classmethod
+    def FlushNaviCache(cls):
+        """Clear the navigation cache"""
+        cls._poscache.Clear()
+
+    @classmethod
+    def GetNaviCacheSize(cls):
+        return cls._poscache.GetSize()
+
     def GetBook(self):
         """Returns the current book used by this object
         @return: path to book used by this manager
 
         """
-        return self._book
+        return self._book        
 
-    def GetNextNaviPos(self, fname=None):
+    @classmethod
+    def GetNextNaviPos(cls, fname=None):
         """Get the next stored navigation position
         The optional fname parameter will get the next found position for
         the given file.
         @param fname: filename (note currently not supported)
         @return: int or None
+        @note: fname is currently not used
 
         """
-        if not len(self._poscache):
-            return None
+        item = cls._poscache.GetNextItem()
+        return item
 
-        # If at end wrap to begining
-        if self._cpos >= len(self._poscache):
-            self._cpos = 0
-
-        rval = None
-        # Get the position at the current history position marker
-        rval = self._poscache[self._cpos]
-        self._cpos += 1
-
-    def GetPreviousNaviPos(self, fname=None):
+    @classmethod
+    def GetPreviousNaviPos(cls, fname=None):
         """Get the last stored navigation position
         The optional fname parameter will get the last found position for
         the given file.
         @param fname: filename (note currently not supported)
         @return: int or None
+        @note: fname is currently not used
 
         """
-        if not len(self._poscache):
-            return None
-
-        # If past begining wrap to the end
-        if self._cpos < 0:
-            self._cpos = len(self._cpos) - 1
-
-        rval = None
-#        if fname is None:
-        # Get the position at the current history position marker
-        rval = self._poscache[self._cpos]
-        self._cpos -= 1
-#        else:
-#            # Get the last known position for the given file
-#            idx = 0
-#            for name, pos in reversed(self._poscache[:self._cpos]):
-#                if name == fname:
-#                    rval = pos
+        item = cls._poscache.GetPreviousItem()
+        return item
 
     def GetPos(self, name):
         """Get the position record for a given filename
@@ -187,7 +192,7 @@ class DocPositionMgr(object):
 
             for line in lines:
                 line = line.strip()
-                vals = line.split(u'=')
+                vals = line.rsplit(u'=', 1)
                 if len(vals) != 2 or not os.path.exists(vals[0]):
                     continue
 
@@ -202,6 +207,21 @@ class DocPositionMgr(object):
             util.Log("[docpositionmgr][info] successfully loaded book")
             return True
 
+    @classmethod
+    def PeekNavi(cls, pre=False):
+        """Peek into the navigation cache
+        @param pre: bool
+
+        """
+        if pre:
+            if cls._poscache.HasPrevious():
+                print "DO PREVIOUS!!"
+                return cls._poscache.PeekPrevious()
+        else:
+            if cls._poscache.HasNext():
+                return cls._poscache.PeekNext()
+        return None, None
+
     def WriteBook(self):
         """Writes the collection of files=pos to the config file
         @postcondition: in memory doc data is written out to disk
