@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 60581 2009-05-10 02:56:00Z CJP $"
-__revision__ = "$Revision: 60581 $"
+__svnid__ = "$Id: ed_stc.py 63656 2010-03-08 23:58:21Z CJP $"
+__revision__ = "$Revision: 63656 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -54,6 +54,30 @@ SPACECHARS = " \t\r\n"
 NONSPACE = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_"
 OPERATORS = "./\?[]{}<>!@#$%^&*():=-+\"';,"
 
+#-------------------------------------------------------------------------#
+
+def jumpaction(func):
+    """Decorator method to notify clients about jump actions"""
+    def WrapJump(*args, **kwargs):
+        stc = args[0]
+        pos = stc.GetCurrentPos()
+        line = stc.GetCurrentLine()
+        func(*args, **kwargs)
+        cpos = stc.GetCurrentPos()
+        cline = stc.GetCurrentLine()
+        fname = stc.GetFileName()
+
+        mdata = dict(fname=fname,
+                     prepos=pos, preline=line,
+                     lnum=cline, pos=cpos)
+        tlw = stc.GetTopLevelParent()
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_POS_JUMPED, mdata, tlw.GetId()) 
+
+    WrapJump.__name__ = func.__name__
+    WrapJump.__doc__ = func.__doc__
+    return WrapJump
+
+
 #-------------------------------------------------------------------------#
 
 class EditraStc(ed_basestc.EditraBaseStc):
@@ -67,7 +91,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
                  style=0, use_dt=True):
         """Initializes a control and sets the default objects for
         Tracking events that occur in the control.
-        @keyword use_dt: wheter to use a drop target or not
+        @keyword use_dt: whether to use a drop target or not
 
         """
         ed_basestc.EditraBaseStc.__init__(self, parent, id_, pos, size, style)
@@ -115,6 +139,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         #self.Bind(wx.stc.EVT_STC_MACRORECORD, self.OnRecordMacro)
         self.Bind(wx.stc.EVT_STC_MARGINCLICK, self.OnMarginClick)
         self.Bind(wx.stc.EVT_STC_UPDATEUI, self.OnUpdateUI)
+        self.Bind(wx.stc.EVT_STC_USERLISTSELECTION, self.OnUserListSel)
         self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
         self.Bind(wx.EVT_CHAR, self.OnChar)
         self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
@@ -215,23 +240,6 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.EndUndoAction()
 
     #---- Begin Function Definitions ----#
-    def AddLine(self, before=False, indent=False):
-        """Add a new line to the document
-        @keyword before: whether to add the line before current pos or not
-        @keyword indent: autoindent the new line
-        @postcondition: a new line is added to the document
-
-        """
-        if before:
-            self.LineUp()
-
-        self.LineEnd()
-
-        if indent:
-            self.AutoIndent()
-        else:
-            self.InsertText(self.GetCurrentPos(), self.GetEOLChar())
-            self.LineDown()
 
     def Bookmark(self, action):
         """Handles bookmark actions
@@ -264,6 +272,16 @@ class EditraStc(ed_basestc.EditraBaseStc):
         if mark != -1:
             self.GotoLine(mark)
 
+    # TODO: DO NOT use these methods anywhere new they will be removed soon
+    def ShowCommandBar(self):
+        """Open the command bar"""
+        self.GetTopLevelParent().GetEditPane().ShowCommandControl(ed_glob.ID_COMMAND)
+
+    def ShowFindBar(self):
+        """Open the quick-find bar"""
+        self.GetTopLevelParent().GetEditPane().ShowCommandControl(ed_glob.ID_QUICK_FIND)
+    # END TODO
+
     def GetBookmarks(self):
         """Gets a list of all lines containing bookmarks
         @return: list of line numbers
@@ -303,7 +321,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
     def Configure(self):
         """Configures the editors settings by using profile values
-        @postcondition: all profile dependant attributes are configured
+        @postcondition: all profile dependent attributes are configured
 
         """
 #        self.SetControlCharSymbol(172)
@@ -324,10 +342,10 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.ToggleAutoIndent(_PGET('AUTO_INDENT'))
         self.ToggleBracketHL(_PGET('BRACKETHL'))
         self.ToggleLineNumbers(_PGET('SHOW_LN'))
-        self.SetViEmulationMode(_PGET('VI_EMU'))
+        self.SetViEmulationMode(_PGET('VI_EMU'), _PGET('VI_NORMAL_DEFAULT'))
         self.SetViewEdgeGuide(_PGET('SHOW_EDGE'))
         self.EnableAutoBackup(_PGET('AUTOBACKUP'))
-        # NOTE: disabled because it is more annoying than it is benificial.
+        # NOTE: disabled because it is more annoying than it is beneficial.
 #        self.SetEndAtLastLine(False)
 
     def ConvertCase(self, upper=False):
@@ -435,8 +453,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
             column = linelen
         self.GotoPos(lstart + column)
 
+    @jumpaction
     def GotoLine(self, line):
-        """Move caret to begining given line number
+        """Move caret to beginning given line number
         @param line: line to go to (int)
 
         """
@@ -452,14 +471,33 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetYCaretPolicy(wx.stc.STC_CARET_EVEN, 0)
         self.PostPositionEvent()
 
+    @jumpaction
     def GotoPos(self, pos):
         """Override StyledTextCtrl.GotoPos
-        @param pos: position in buffer to move carat to (int)
+        @param pos: position in buffer to move caret to (int)
 
         """
         super(EditraStc, self).GotoPos(pos)
         self.PostPositionEvent()
 
+    def SetCaretPos(self, pos):
+        """Set the caret position without posting jump events
+        @param pos: position to go to
+
+        """
+        super(EditraStc, self).GotoPos(pos)
+        self.PostPositionEvent()
+
+    def GotoIndentPos(self, line=None):
+        """Move the caret to the end of the indentation
+        on the given line.
+        @param line: line to go to
+
+        """
+        if line is None:
+            line = self.GetCurrentLine()
+        self.GotoPos(self.GetLineIndentPosition(line))
+
     def SetCurrentCol(self, column):
         """Set the current column position on the currently line
         extending the selection.
@@ -474,16 +512,6 @@ class EditraStc(ed_basestc.EditraBaseStc):
             column = linelen
         self.SetCurrentPos(lstart + column)
 
-    def GotoIndentPos(self, line=None):
-        """Move the caret to the end of the indentation
-        on the given line.
-        @param line: line to go to
-
-        """
-        if line is None:
-            line = self.GetCurrentLine()
-        self.GotoPos(self.GetLineIndentPosition(line))
-
     def DeleteForward(self):
         """Delete the selection, or if there is no selection, then
         delete the character to the right of the cursor.
@@ -519,6 +547,10 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
         # If the file is different than the last save point make the backup.
         bkupmgr = ebmlib.FileBackupMgr(None, u"%s.edbkup")
+        path = _PGET('AUTOBACKUP_PATH', default=u"")
+        if path and os.path.exists(path):
+            bkupmgr.SetBackupDirectory(path)
+
         if not self._backup_done and \
            (not bkupmgr.HasBackup(fname) or bkupmgr.IsBackupNewer(fname)):
             writer = bkupmgr.GetBackupWriter(self.File)
@@ -578,11 +610,11 @@ class EditraStc(ed_basestc.EditraBaseStc):
             evt.Skip()
 
     def OnChar(self, evt):
-        """Handles Char events that arent caught by the
+        """Handles Char events that aren't caught by the
         KEY_DOWN event.
         @param evt: event that called this handler
         @type evt: wx.EVT_CHAR
-        @todo: autocomp/calltip lookup can be very cpu intesive it may
+        @todo: autocomp/calltip lookup can be very cpu intensive it may
                be better to try and process it on a separate thread to
                prevent a slow down in the input of text into the buffer
 
@@ -602,14 +634,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
             return
 
         elif key_code in cmpl.GetAutoCompKeys():
-            if self.AutoCompActive():
-                self.AutoCompCancel()
+            self.HidePopups()
 
-            if self.CallTipActive():
-                self.CallTipCancel()
+            uchr = unichr(key_code)
+            command = self.GetCommandStr() + uchr
+            self.PutText(uchr)
 
-            command = self.GetCommandStr() + unichr(key_code)
-            self.AddText(unichr(key_code))
             if self._config['autocomp']:
                 self.ShowAutoCompOpt(command)
 
@@ -624,8 +654,11 @@ class EditraStc(ed_basestc.EditraBaseStc):
         elif key_code in cmpl.GetCallTipKeys():
             if self.AutoCompActive():
                 self.AutoCompCancel()
+
             command = self.GetCommandStr()
-            self.AddText(unichr(key_code))
+            uchr = unichr(key_code)
+            self.PutText(uchr)
+
             if self._config['autocomp']:
                 self.ShowCallTip(command)
 
@@ -650,16 +683,21 @@ class EditraStc(ed_basestc.EditraBaseStc):
         """
         evt.Skip()
         self.PostPositionEvent()
+        tlw = self.GetTopLevelParent()
         ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_KEYUP,
-                           (evt.GetPositionTuple(), evt.GetKeyCode()))
+                           (evt.GetPositionTuple(), evt.GetKeyCode()),
+                           tlw.GetId())
 
     def PostPositionEvent(self):
         """Post an event to update the status of the line/column"""
-        pos = self.GetPos()
-        msg = _("Line: %(lnum)d  Column: %(cnum)d") % dict(lnum=pos[0], cnum=pos[1])
+        line, column = self.GetPos()
+        pinfo = dict(lnum=line, cnum=column)
+        msg = _("Line: %(lnum)d  Column: %(cnum)d") % pinfo
         nevt = ed_event.StatusEvent(ed_event.edEVT_STATUS, self.GetId(),
                                     msg, ed_glob.SB_ROWCOL)
-        wx.PostEvent(self.GetTopLevelParent(), nevt)
+        tlw = self.GetTopLevelParent()
+        wx.PostEvent(tlw, nevt)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_POS_CHANGED, pinfo, tlw.GetId())
 
     def OnRecordMacro(self, evt):
         """Records macro events
@@ -706,6 +744,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.WordLeftExtend()
             self.SetCurrentPos(self.GetCurrentPos() + len(self.GetEOLChar()))
 
+    @jumpaction
     def OnLeftUp(self, evt):
         """Set primary selection and inform mainwindow that cursor position
         has changed.
@@ -726,7 +765,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         pid = self.GetTopLevelParent().GetId()
         if evt.GetState() == ed_txt.FL_STATE_READING:
             if evt.HasText():
-                # TODO: gauge gauge updates working properly
+                # TODO: get gauge updates working properly
 #                sb = self.GetTopLevelParent().GetStatusBar()
 #                gauge = sb.GetGauge()
 #                gauge.SetValue(evt.GetProgress())
@@ -776,6 +815,15 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.VertEdit.OnUpdateUI(evt)
         evt.Skip()
 
+    def OnUserListSel(self, evt):
+        """Callback hook for userlist selections"""
+        mdata = dict(ltype=evt.GetListType(),
+                     text=evt.GetText(),
+                     stc=self)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_USERLIST_SEL, mdata,
+                           context=self.GetTopLevelParent().GetId())
+        evt.Skip()
+
     def OnMarginClick(self, evt):
         """Open and Close Folders as Needed
         @param evt: event that called this handler
@@ -798,7 +846,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
                     elif evt.GetControl() or \
                         (wx.Platform == '__WXMAC__' and evt.GetAlt()):
                         # Contract all subnodes of clicked one
-                        # Note: using Alt as Ctrl can not be recieved for
+                        # Note: using Alt as Ctrl can not be received for
                         # clicks on mac (Scintilla Bug).
                         if self.GetFoldExpanded(line_clicked):
                             self.SetFoldExpanded(line_clicked, False)
@@ -884,6 +932,14 @@ class EditraStc(ed_basestc.EditraBaseStc):
                 line = line + 1
         return line
 
+    def ExpandAll(self):
+        """Expand all folded code blocks"""
+        line_count = self.GetLineCount()
+        for line_num in xrange(line_count):
+            if self.GetFoldLevel(line_num) & wx.stc.STC_FOLDLEVELHEADERFLAG:
+                if not self.GetFoldExpanded(line_num):
+                    self.Expand(line_num, True)
+
     def FindLexer(self, set_ext=u''):
         """Sets Text Controls Lexer Based on File Extension
         @param set_ext: explicit extension to use in search
@@ -952,11 +1008,8 @@ class EditraStc(ed_basestc.EditraBaseStc):
                     ed_glob.ID_ADD_BM    : self.Bookmark,
                     ed_glob.ID_DEL_ALL_BM : self.Bookmark}
 
-        if self.CallTipActive():
-            self.CallTipCancel()
-
-        if self.AutoCompActive():
-            self.AutoCompCancel()
+        # Hide autocomp popups
+        self.HidePopups()
 
         if e_obj.GetClassName() == "wxToolBar" or e_id in e_map:
             if e_id in e_map:
@@ -979,9 +1032,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         elif e_id == ed_glob.ID_WORD_WRAP:
             self.SetWrapMode(not self.GetWrapMode())
         elif e_id == ed_glob.ID_JOIN_LINES:
-            self.SetTargetStart(self.GetSelectionStart())
-            self.SetTargetEnd(self.GetSelectionEnd())
-            self.LinesJoin()
+            self.LinesJoinSelected()
         elif e_id == ed_glob.ID_INDENT_GUIDES:
             self.SetIndentationGuides(not bool(self.GetIndentationGuides()))
         elif e_id == ed_glob.ID_HLCARET_LINE:
@@ -1053,7 +1104,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
                     super(EditraStc, self).SetEOLMode(sel)
                 dlg.Destroy()
             else:
-                # The end of line character is different from the prefered
+                # The end of line character is different from the preferred
                 # user setting for end of line. So change our eol mode to
                 # preserve that of what the document is using.
                 mode = eol_map.get(eol, wx.stc.STC_EOL_LF)
@@ -1111,7 +1162,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         return self.LineFromPosition(self.GetCurrentPos())
 
     def GetEOLModeId(self):
-        """Gets the id of the eol format. Convinience for updating
+        """Gets the id of the eol format. Convenience for updating
         menu ui.
         @return: id of the eol mode of this document
 
@@ -1204,6 +1255,15 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetTargetEnd(self.GetLineEndPosition(eline))
         self.ReplaceTarget(u' '.join(lines))
 
+    def LinesJoinSelected(self):
+        """Similar to LinesJoin, but operates on selection
+        @see: LinesJoin
+
+        """
+        self.SetTargetStart(self.GetSelectionStart())
+        self.SetTargetEnd(self.GetSelectionEnd())
+        self.LinesJoin()
+
     def LineMoveUp(self):
         """Move the current line up"""
         linenum = self.GetCurrentLine()
@@ -1245,7 +1305,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
                 self.InitCompleter()
 
     def SetEOLMode(self, mode):
-        """Sets the EOL mode from a string descript
+        """Sets the EOL mode from a string description
         @param mode: eol mode to set
         @note: overrides StyledTextCtrl.SetEOLMode
 
@@ -1258,15 +1318,17 @@ class EditraStc(ed_basestc.EditraBaseStc):
         mode = mode_map.get(mode, wx.stc.STC_EOL_LF)
         super(EditraStc, self).SetEOLMode(mode)
 
-    def SetViEmulationMode(self, use_vi):
-        """Activate/Deactivate Vi eumulation mode
+    def SetViEmulationMode(self, use_vi, use_normal=False):
+        """Activate/Deactivate Vi emulation mode
         @param use_vi: Turn vi emulation on/off
         @type use_vi: boolean
+        @keyword use_normal: Start in normal mode
+        @type use_normal: boolean
 
         """
         self.key_handler.ClearMode()
         if use_vi:
-            self.key_handler = ViKeyHandler(self)
+            self.key_handler = ViKeyHandler(self, use_normal)
         else:
             self.key_handler = KeyHandler(self)
 
@@ -1378,6 +1440,10 @@ class EditraStc(ed_basestc.EditraBaseStc):
         else:
             self.LOG("[ed_stc][evt] Code Folding Turned Off")
             self._config['folding'] = False
+
+            # Ensure all code blocks have been expanded
+            self.ExpandAll()
+
             self.SetMarginWidth(ed_basestc.FOLD_MARGIN, 0)
 
     def SyntaxOnOff(self, switch=None):
@@ -1429,125 +1495,8 @@ class EditraStc(ed_basestc.EditraBaseStc):
             lineNum = self.GetCurrentLine()
         super(EditraStc, self).ToggleFold(lineNum)
 
-    def MoveCurrentPos(self, offset, extend=False):
-        """Move caret by the given offset
-        @note: only use it for movement within a line
-
-        """
-        if offset > 0:
-            step = self.CharRight #XXX breaks RTL, but is RTL supported?
-            if extend:
-                step = self.CharRightExtend
-        else:
-            step = self.CharLeft
-            if extend:
-                step = self.CharLeftExtend
-
-        #XXX is there a better reliable way than repeating basic movements?
-        repeat = abs(offset)
-        for i in range(repeat):
-            step()
-
-    def _FindChar(self, char, repeat=1, reverse=False, extra_offset=0):
-        """Find the position of the next (ith) 'char' character
-        on the current line
-
-        @note used by vim motions for finding a character on a line (f,F,t,T)
-        @param char: the char to be found
-        @keyword repeat: how many times to repeat the serach
-        @keyword reverse: whether to search backwards
-        @keyword extra_offset: extra offset to be applied to the return value
-
-        @return: offset from caret position
-
-        """
-        text, pos = self.GetCurLine()
-        oldpos = pos
-        if not reverse:
-            for i in range(repeat):
-                pos = text.find(char, pos+1) # pos is on the char itself
-                if pos == -1:
-                    break # -1 + 1 = 0
-        else:
-            for i in range(repeat):
-                pos = text.rfind(char, 0, pos)
-                if pos == -1:
-                    break # TEST not sure but we need this as well
-
-        # if pos == -1 then the char was not found
-        # but offset could be arbitrary
-        # (although should never be other than 0 or -1) so do a bound check
-        newpos = pos + extra_offset
-        if newpos not in range(len(text)):
-            return 0
-
-        return newpos - oldpos;
-
-    def FindNextChar(self, char, repeat=1, extend=False):
-        """Move caret to the next (ith) occurance of char on the current line
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-        offset = self._FindChar(char, repeat)
-        self.MoveCurrentPos(offset, extend)
-
-    def FindPrevChar(self, char, repeat=1, extend=False):
-        """Move caret to the previous (ith) occurance of char on the current line
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-
-        offset = self._FindChar(char, repeat, reverse=True)
-        self.MoveCurrentPos(offset, extend)
-
-    def FindTillNextChar(self, char, repeat=1, extend=False):
-        """Move caret until right before the next ith occurance of char
-
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-        offset = self._FindChar(char, repeat, extra_offset=-1)
-        self.MoveCurrentPos(offset, extend)
-
-    def FindTillPrevChar(self, char, repeat=1, extend=False):
-        """Move caret until right before the previous ith occurance of char
-
-        @note: This is a vim motion
-        @keyword repeat: int determining ith occurance to move to
-        @keyword extend: whether to extend selection or not
-
-        """
-        offset = self._FindChar(char, repeat, reverse=True, extra_offset=-1)
-        self.MoveCurrentPos(offset, extend)
-
-
-    def GetIdentifierUnderCursor(self):
-        """Returns the identifier under the cursor (if any)"""
-        line, pos = self.GetCurLine()
-        while pos > 0: # rewind to first char of identifier
-            c = line[pos-1]
-            if c == '_' or c.isalnum():
-               pos = pos - 1
-            else:
-                break
-
-        start = pos
-        while pos < len(line): # find end of identifier
-            c = line[pos]
-            if c == '_' or c.isalnum():
-               pos = pos + 1
-            else:
-                break
-        return line[start:pos]
-
     def WordLeft(self):
-        """Move caret to begining of previous word
+        """Move caret to beginning of previous word
         @note: override builtin to include extra characters in word
 
         """
@@ -1559,7 +1508,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetWordChars('')
 
     def WordLeftExtend(self):
-        """Extend selection to begining of previous word
+        """Extend selection to beginning of previous word
         @note: override builtin to include extra characters in word
 
         """
@@ -1571,7 +1520,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetWordChars('')
 
     def WordPartLeft(self):
-        """Move the caret left to the next change in capitalization/puncuation
+        """Move the caret left to the next change in capitalization/punctuation
         @note: overrides default function to not count whitespace as words
 
         """
@@ -1581,7 +1530,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             super(EditraStc, self).WordPartLeft()
 
     def WordPartLeftExtend(self):
-        """Extend selection left to the next change in capitalization/puncuation
+        """Extend selection left to the next change in capitalization/punctuation
         @note: overrides default function to not count whitespace as words
 
         """
@@ -1601,7 +1550,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             super(EditraStc, self).WordPartRight()
 
     def WordPartRightEnd(self):
-        """Move caret to end of next change in capitalization/puncuation
+        """Move caret to end of next change in capitalization/punctuation
         @postcondition: caret is moved
 
         """
@@ -1612,7 +1561,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.CharLeft()
 
     def WordPartRightEndExtend(self):
-        """Extend selection to end of next change in capitalization/puncuation
+        """Extend selection to end of next change in capitalization/punctuation
         @postcondition: selection is extended
 
         """
@@ -1623,7 +1572,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.CharLeftExtend()
 
     def WordPartRightExtend(self):
-        """Extend selection to start of next change in capitalization/puncuation
+        """Extend selection to start of next change in capitalization/punctuation
         @postcondition: selection is extended
 
         """
@@ -1633,7 +1582,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
             super(EditraStc, self).WordPartRightExtend()
 
     def WordRight(self):
-        """Move caret to begining of next word
+        """Move caret to beginning of next word
         @note: override builtin to include extra characters in word
 
         """
@@ -1657,7 +1606,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
         self.SetWordChars('')
 
     def WordRightExtend(self):
-        """Extend selection to begining of next word
+        """Extend selection to beginning of next word
         @note: override builtin to include extra characters in word
 
         """
@@ -1714,7 +1663,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
                 return False, msg
             else:
                 self.GotoPos(cpos)
-                ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, self.GetFileName())
+                context = self.GetTopLevelParent().GetId()
+                ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED,
+                                   self.GetFileName(), context)
                 return True, ''
         else:
             self.LOG("[ed_stc][err] %s does not exists, cant reload." % cfile)
@@ -1754,8 +1705,9 @@ class EditraStc(ed_basestc.EditraBaseStc):
         """
         result = True
         try:
+            tlw_id = self.GetTopLevelParent().GetId()
             ed_msg.PostMessage(ed_msg.EDMSG_FILE_SAVE,
-                               (path, self.GetLangId()))
+                               (path, self.GetLangId()), tlw_id)
             self.File.SetPath(path)
             self.LOG("[ed_stc][info] Writing file %s, with encoding %s" % \
                      (path, self.File.GetEncoding()))
@@ -1769,7 +1721,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
                               style=wx.OK|wx.CENTER|wx.ICON_WARNING)
                 return True
             else:
-                self.File.Write(self.GetText())
+                if not self.File.IsRawBytes():
+                    self.File.Write(self.GetText())
+                else:
+                    nchars = self.GetTextLength()
+                    txt = self.GetStyledText(0, nchars)[0:nchars*2:2]
+                    self.File.Write(txt)
         except Exception, msg:
             result = False
             self.LOG("[ed_stc][err] There was an error saving %s" % path)
@@ -1782,19 +1739,23 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.SetFileName(path)
 
         wx.CallAfter(ed_msg.PostMessage,
-                    ed_msg.EDMSG_FILE_SAVED,
-                    (path, self.GetLangId()))
+                     ed_msg.EDMSG_FILE_SAVED,
+                     (path, self.GetLangId()),
+                     tlw_id)
 
         return result
 
     def ConfigureLexer(self, file_ext):
-        """Sets Lexer and Lexer Keywords for the specifed file extension
+        """Sets Lexer and Lexer Keywords for the specified file extension
         @param file_ext: a file extension to configure the lexer from
 
         """
         super(EditraStc, self).ConfigureLexer(file_ext)
 
         # Notify that lexer has changed
+        self.LOG("[ed_stc][info] Lexer change notification for context %d" %
+                 self.GetTopLevelParent().GetId())
         ed_msg.PostMessage(ed_msg.EDMSG_UI_STC_LEXER,
-                           (self.GetFileName(), self.GetLangId()))
+                           (self.GetFileName(), self.GetLangId()),
+                           context=self.GetTopLevelParent().GetId())
         return True
