@@ -2,7 +2,7 @@
 # Purpose:      Classes for parameter introduction
 # Author:       Roman Rolinsky <rolinsky@mema.ucl.ac.be>
 # Created:      22.08.2001
-# RCS-ID:       $Id: params.py 52400 2008-03-09 03:29:56Z RD $
+# RCS-ID:       $Id: params.py 64107 2010-04-22 14:05:36Z ROL $
 
 '''
 Visual C{Param*} classes for populating C{AtrtibutePanel} with attribute editing
@@ -82,7 +82,9 @@ class PPanel(wx.Panel):
         Presenter.setApplied(False)
         evt.Skip()
     def OnKillFocus(self, evt):
-        if g.conf.autoRefresh and g.conf.autoRefreshPolicy == AUTO_REFRESH_POLICY_FOCUS:
+        # Refresh test window if auto refresh policy on focus
+        if Listener.testWin.IsShown() and g.conf.autoRefresh and \
+                g.conf.autoRefreshPolicy == AUTO_REFRESH_POLICY_FOCUS:
             wx.CallAfter(Presenter.refreshTestWin)
         evt.Skip()
 
@@ -392,8 +394,9 @@ class ParamText(PPanel):
     '''Text attribute.'''
     textWidth = -1
     proportion = 0
-    def __init__(self, parent, name, style=0, **kargs):
+    def __init__(self, parent, name, **kargs):
         PPanel.__init__(self, parent, name)
+        style = kargs.pop('style', 0)
         textWidth = kargs.pop('textWidth', self.textWidth)
         option = kargs.pop('proportion', self.proportion)
         if textWidth == -1: option = 1
@@ -422,7 +425,7 @@ ParamPosSize = MetaParamText(80)
 class ParamComment(ParamText):
     '''Comment node editing.'''
     def __init__(self, parent, name):
-        ParamText.__init__(self, parent, name, 330,
+        ParamText.__init__(self, parent, name, textWidth=330,
                            style=wx.TE_PROCESS_ENTER)
 
 class ContentDialog(wx.Dialog):
@@ -708,7 +711,8 @@ class CheckBox(PPanel):
         Presenter.setApplied(False)
         if Presenter.panelIsDirty():
             Presenter.registerUndoEdit()
-        if g.conf.autoRefresh and g.conf.autoRefreshPolicy == AUTO_REFRESH_POLICY_FOCUS:
+        if Listener.testWin.IsShown() and g.conf.autoRefresh and \
+                g.conf.autoRefreshPolicy == AUTO_REFRESH_POLICY_FOCUS:
             Listener.testWin.isDirty = True
             wx.CallAfter(Presenter.refreshTestWin)
         evt.Skip()
@@ -950,9 +954,11 @@ class StylePanel(wx.Panel):
 
     def OnCheck(self, evt):
         Presenter.setApplied(False)
-        if g.conf.autoRefresh and g.conf.autoRefreshPolicy == AUTO_REFRESH_POLICY_FOCUS:
+        if Listener.testWin.IsShown() and g.conf.autoRefresh and \
+                g.conf.autoRefreshPolicy == AUTO_REFRESH_POLICY_FOCUS:
             Listener.testWin.isDirty = True
             wx.CallAfter(Presenter.refreshTestWin)
+        evt.Skip()
 
 #############################################################################
 
